-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose:
-- . Wrapper containing dp_offload_tx and design-specific header fields
-- FIXME 2) The *entire* UDP payload should be little endian; including the 'id' and 'flag' fields.

LIBRARY IEEE, common_lib, work, mm_lib, unb1_board_lib, dp_lib, tech_tse_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE tech_tse_lib.tech_tse_pkg.ALL;

ENTITY apertif_unb1_correlator_vis_offload IS
  PORT (
    mm_rst                         : IN  STD_LOGIC;
    mm_clk                         : IN  STD_LOGIC; 
    
    dp_rst                         : IN  STD_LOGIC;
    dp_clk                         : IN  STD_LOGIC;  

    reg_dp_offload_tx_mosi         : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_dp_offload_tx_miso         : OUT t_mem_miso := c_mem_miso_rst;
    reg_dp_offload_tx_hdr_dat_mosi : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_dp_offload_tx_hdr_dat_miso : OUT t_mem_miso := c_mem_miso_rst;                                    

    snk_in                         : IN  t_dp_sosi; 
    snk_out                        : OUT t_dp_siso;              

    src_out_arr                    : OUT t_dp_sosi_arr(0 DOWNTO 0); 
    src_in_arr                     : IN  t_dp_siso_arr(0 DOWNTO 0) := (OTHERS=>c_dp_siso_rdy);

    ID                             : IN  STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0)
  );
END apertif_unb1_correlator_vis_offload;


ARCHITECTURE wrap OF apertif_unb1_correlator_vis_offload IS

                                       -- Word align + Eth + IP + UDP + ID + Flags
  CONSTANT c_nof_hdr_fields : NATURAL :=           1 +   3 + 12 +   4 +  6 + 8 ;    -- 34 fields
                     -- Numer of bytes:            2 + <-----42-----> + 16 + 8*24/8 -- 2 + 42 + 40 = 84 bytes = 672 bits

  CONSTANT c_hdr_field_arr  : t_common_field_arr(c_nof_hdr_fields-1 DOWNTO 0) := ( ( field_name_pad("word_align"                          ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("eth_dst_mac"                         ), "  ", 48, field_default(x"0030482D82A1") ),
                                                                                   ( field_name_pad("eth_src_mac"                         ), "  ", 48, field_default(0) ),
                                                                                   ( field_name_pad("eth_type"                            ), "  ", 16, field_default(x"800") ),
                                                                                   ( field_name_pad("ip_version"                          ), "  ",  4, field_default(4) ),
                                                                                   ( field_name_pad("ip_header_length"                    ), "  ",  4, field_default(5) ),
                                                                                   ( field_name_pad("ip_services"                         ), "  ",  8, field_default(0) ),
                                                                                   ( field_name_pad("ip_total_length"                     ), "  ", 16, field_default(2468) ),
                                                                                   ( field_name_pad("ip_identification"                   ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("ip_flags"                            ), "  ",  3, field_default(2) ),
                                                                                   ( field_name_pad("ip_fragment_offset"                  ), "  ", 13, field_default(0) ),
                                                                                   ( field_name_pad("ip_time_to_live"                     ), "  ",  8, field_default(127) ),
                                                                                   ( field_name_pad("ip_protocol"                         ), "  ",  8, field_default(17) ),
                                                                                   ( field_name_pad("ip_header_checksum"                  ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("ip_src_addr"                         ), "  ", 32, field_default(0) ),
                                                                                   ( field_name_pad("ip_dst_addr"                         ), "  ", 32, field_default(x"0A6300FE") ),
                                                                                   ( field_name_pad("udp_src_port"                        ), "  ", 16, field_default(4000) ), 
                                                                                   ( field_name_pad("udp_dst_port"                        ), "  ", 16, field_default(4000) ), 
                                                                                   ( field_name_pad("udp_total_length"                    ), "  ", 16, field_default(2448) ),
                                                                                   ( field_name_pad("udp_checksum"                        ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("id_marker_byte"                      ), "  ",  8, field_default(65) ),
                                                                                   ( field_name_pad("id_format_version"                   ), "  ",  8, field_default(2) ),  
                                                                                   ( field_name_pad("id_beamlet_index"                    ), "  ", 16, field_default(x"BEA1") ),
                                                                                   ( field_name_pad("id_channel_index"                    ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("id_reserved"                         ), "  ", 16, field_default(0) ),
                                                                                   ( field_name_pad("id_timestamp"                        ), "  ", 64, field_default(0) ),
                                                                                   ( field_name_pad("flags_crc_error"                     ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_no_input_present"              ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_uploading_weights"             ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_noise_source_enabled"          ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_telescope_pointing_off"        ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_antenna_broken"                ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_reserved_0"                    ), "  ", 24, field_default(0) ),
                                                                                   ( field_name_pad("flags_reserved_1"                    ), "  ", 24, field_default(0) ) ); 

  CONSTANT c_hdr_field_sel         : STD_LOGIC_VECTOR(c_nof_hdr_fields-1 DOWNTO 0)  :=             "1101" &  -- Eth header fields: eth_src_mac from data path (others use MM default value).
                                                                                           "111111111101" &  -- IP  header fields: ip_src_addr from data path (others use MM default value).
                                                                                                   "1111" &  -- UDP header fields
--                                                                                               "110010" &  -- ID  header fields: MM controlled: marker byte, format version, reserved. From data path: beamlet+channel index and timestamp.
                                                                                                 "111010" &  -- ID FIXME: Now using defaults for beamlet index for testing.
                                                                                               "00111111";   -- Flag fields: crc_error and no_input_present from data path, others via MM.

  CONSTANT c_nof_hdr_words              : NATURAL := field_slv_len(c_hdr_field_arr)/c_tech_tse_data_w;
  CONSTANT c_nof_offload_streams        : NATURAL := 1;

  CONSTANT c_nof_words_per_block        : NATURAL := 2*300; -- factor 2 because of 64b->32b interleaving.
  CONSTANT c_nof_blocks_per_packet      : NATURAL := 1;

  SIGNAL dp_offload_tx_snk_in_arr       : t_dp_sosi_arr(c_nof_offload_streams-1 DOWNTO 0);
  SIGNAL dp_offload_tx_snk_out_arr      : t_dp_siso_arr(c_nof_offload_streams-1 DOWNTO 0);

  SIGNAL hdr_fields_in_arr              : t_slv_1024_arr(c_nof_offload_streams-1 DOWNTO 0);
  SIGNAL id_backplane                   : STD_LOGIC_VECTOR(c_byte_w-1 DOWNTO 0);
  SIGNAL id_chip                        : STD_LOGIC_VECTOR(c_byte_w-1 DOWNTO 0);

BEGIN

  --------------------------------------------------------------------------------------- 
  -- Swap the bytes in the 32b words to make them little-endian
  ---------------------------------------------------------------------------------------
  p_connect : PROCESS(snk_in)
  BEGIN
    -- Control
    dp_offload_tx_snk_in_arr(0)                                      <= snk_in;
    -- Data: re&im concatenated so real part is put on the line first: re,im,re,im, ..
    dp_offload_tx_snk_in_arr(0).data(  c_byte_w-1 DOWNTO          0) <= snk_in.data(4*c_byte_w-1 DOWNTO 3*c_byte_w);
    dp_offload_tx_snk_in_arr(0).data(2*c_byte_w-1 DOWNTO   c_byte_w) <= snk_in.data(3*c_byte_w-1 DOWNTO 2*c_byte_w);
    dp_offload_tx_snk_in_arr(0).data(3*c_byte_w-1 DOWNTO 2*c_byte_w) <= snk_in.data(2*c_byte_w-1 DOWNTO   c_byte_w);
    dp_offload_tx_snk_in_arr(0).data(4*c_byte_w-1 DOWNTO 3*c_byte_w) <= snk_in.data(  c_byte_w-1 DOWNTO          0);
  END PROCESS;

  --------------------------------------------------------------------------------------- 
  -- dp_offload_tx
  ---------------------------------------------------------------------------------------
  snk_out <= dp_offload_tx_snk_out_arr(0);

  u_dp_offload_tx : ENTITY dp_lib.dp_offload_tx
  GENERIC MAP (
    g_nof_streams               => c_nof_offload_streams,
    g_data_w                    => c_tech_tse_data_w,
    g_use_complex               => FALSE,
    g_max_nof_words_per_block   => c_nof_words_per_block,
    g_def_nof_words_per_block   => c_nof_words_per_block,
    g_max_nof_blocks_per_packet => c_nof_blocks_per_packet,
    g_def_nof_blocks_per_packet => c_nof_blocks_per_packet,
    g_hdr_field_arr             => c_hdr_field_arr,
    g_hdr_field_sel             => c_hdr_field_sel
   )
  PORT MAP (
    mm_rst                => mm_rst,
    mm_clk                => mm_clk,
    
    dp_rst                => dp_rst,
    dp_clk                => dp_clk,

    reg_mosi              => reg_dp_offload_tx_mosi,
    reg_miso              => reg_dp_offload_tx_miso,

    reg_hdr_dat_mosi      => reg_dp_offload_tx_hdr_dat_mosi,
    reg_hdr_dat_miso      => reg_dp_offload_tx_hdr_dat_miso,
 
    snk_in_arr            => dp_offload_tx_snk_in_arr,
    snk_out_arr           => dp_offload_tx_snk_out_arr,

    src_out_arr           => src_out_arr,
    src_in_arr            => src_in_arr,

    hdr_fields_in_arr     => hdr_fields_in_arr
  );

  --------------------------------------------------------------------------------------- 
  -- Extract the backplane number from ID
  ---------------------------------------------------------------------------------------
  id_backplane <= RESIZE_UVEC(ID(7 DOWNTO 3), c_byte_w);
  id_chip      <= RESIZE_UVEC(ID(2 DOWNTO 0), c_byte_w);

  --------------------------------------------------------------------------------------- 
  -- Assign DP record fields and ID to header fields
  ---------------------------------------------------------------------------------------
  gen_slv_hard_fields : FOR i IN 0 TO c_nof_offload_streams-1 GENERATE
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "eth_src_mac"        ) DOWNTO field_lo(c_hdr_field_arr, "eth_src_mac"       )) <= x"00228608" & id_backplane & id_chip;
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "udp_src_port"       ) DOWNTO field_lo(c_hdr_field_arr, "udp_src_port"      )) <= x"D0" & ID;
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "udp_dst_port"       ) DOWNTO field_lo(c_hdr_field_arr, "udp_dst_port"      )) <= x"D0" & ID;
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "ip_src_addr"        ) DOWNTO field_lo(c_hdr_field_arr, "ip_src_addr"       )) <= x"0A63" & id_backplane & INCR_UVEC(id_chip, 1);

    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "id_channel_index"   ) DOWNTO field_lo(c_hdr_field_arr, "id_channel_index"  )) <= snk_in.channel(15 DOWNTO 0);
    hdr_fields_in_arr(i)(field_hi(c_hdr_field_arr, "id_timestamp"       ) DOWNTO field_lo(c_hdr_field_arr, "id_timestamp"      )) <= snk_in.bsn;
  END GENERATE;

END wrap;
