from common_dsp import *
from file_io import *

REC_FILE = os.environ['RADIOHDL']+'/applications/apertif/designs/apertif_unb1_correlator/tb/rec/correlator_src_out_arr0.rec'
USE_NEW_REC_FILE = False 

NOF_INPUTS = 24
NOF_CHANNELS = 64
NOF_VISIBILITIES = (NOF_INPUTS*(NOF_INPUTS+1))/2
NOF_LINES_IN_REC_FILE_TO_VERIFY = NOF_CHANNELS*NOF_VISIBILITIES-1 #FIXME: Last line is written when sim is stopped. Hence -1.
CORRELATOR_INPUT_WIDTH=9
CORRELATOR_INTEGRATION_PERIOD = 12207

CORRELATOR_MULT_OUTPUT_WIDTH = 2*CORRELATOR_INPUT_WIDTH
CORRELATOR_OUTPUT_COMPLEX_WIDTH = CORRELATOR_MULT_OUTPUT_WIDTH + ceil_log2(CORRELATOR_INTEGRATION_PERIOD)

# Wait until file exists
do_until(file_exists, True, s_timeout=900, filename=REC_FILE)

if USE_NEW_REC_FILE==True:
    # Wait until file is modified
    print 'Waiting for %s to be modified...' %REC_FILE
    now = time.time()
    do_until_gt(file_mod_time, now, s_timeout=900, filename=REC_FILE)
    print '%s modified. Waiting for %d lines to be written...' %(REC_FILE, NOF_LINES_IN_REC_FILE_TO_VERIFY)
    do_until_gt(file_linecount, NOF_LINES_IN_REC_FILE_TO_VERIFY, s_timeout=3600, filename=REC_FILE)

# Read test bench output file written by dp_stream_rec_play.vhd
correlator_src_out_arr = rec_file_to_complex(REC_FILE, complex_width=CORRELATOR_OUTPUT_COMPLEX_WIDTH)

# Splits the list into sublists of NOF_CHANNELS long: one list of NOF_CHANNELS per integration period.
correlator_src_out_arr = split_list(correlator_src_out_arr, NOF_CHANNELS)

# Plot integration period 0
for integration_period in correlator_src_out_arr[0:1]:
    # The correlator outputs channels in the same order as the FFT, so reorder first:
    integration_period = fft_reorder(integration_period)
    # Plot the phase shifts
    plot_phase_shifts(integration_period, NOF_VISIBILITIES)


