------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------
LIBRARY IEEE, common_lib, unb1_board_lib, dp_lib, technology_lib, io_ddr_lib, eth_lib, tech_tse_lib, bf_lib, tech_ddr_lib, tr_10GbE_lib, reorder_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE tech_ddr_lib.tech_ddr_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE tech_tse_lib.tech_tse_pkg.ALL;
USE tech_tse_lib.tb_tech_tse_pkg.ALL;  
USE reorder_lib.reorder_pkg.ALL;
USE bf_lib.bf_pkg.ALL;

ENTITY apertif_unb1_fn_beamformer IS
  GENERIC (
    g_design_name : STRING  := "apertif_unb1_fn_beamformer"; --"apertif_unb1_fn_beamformer";
    g_design_note : STRING  := "revision info"; --"UNUSED";
    g_sim         : BOOLEAN := FALSE; --Overridden by TB
    g_sim_model   : BOOLEAN := FALSE; -- Use fast DDR3 model
    g_sim_unb_nr  : NATURAL := 0;
    g_sim_node_nr : NATURAL := 0;
    g_stamp_date  : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time  : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn   : NATURAL := 0;  -- SVN revision    -- set by QSF
    g_bf          : t_c_bf  := c_bf
  );
  PORT (
   -- GENERAL
    CLK                    : IN    STD_LOGIC; -- System Clock
    PPS                    : IN    STD_LOGIC; -- System Sync
    WDI                    : OUT   STD_LOGIC; -- Watchdog Clear
    INTA                   : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB                   : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION                : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID                     : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO                 : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);

    -- I2C Interface to Sensors
    sens_sc                : INOUT STD_LOGIC;
    sens_sd                : INOUT STD_LOGIC;

    -- 1GbE Control Interface
    ETH_clk                : IN    STD_LOGIC;
    ETH_SGIN               : IN    STD_LOGIC;
    ETH_SGOUT              : OUT   STD_LOGIC;
    
    -- Transceiver clocks
    SA_CLK                 : IN  STD_LOGIC := '0';  -- TR clock BN-BI (tr_back) / SI_FN (tr_front)
    SB_CLK                 : IN  STD_LOGIC := '0';  -- TR clock FN-BN (tr_mesh)

    -- Mesh Serial I/O
    FN_BN_0_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_0_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_1_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_1_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_2_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_2_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_3_TX             : OUT STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0);
    FN_BN_3_RX             : IN  STD_LOGIC_VECTOR(c_unb1_board_tr_mesh.bus_w-1 DOWNTO 0) := (OTHERS=>'0');

    -- Front Serial I/O
    SI_FN_0_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_0_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);

    SI_FN_0_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0); -- (0 = LASI; 1=MDC; 2=MDIO)
    SI_FN_1_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_2_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_3_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_RSTN             : OUT   STD_LOGIC := '1'; -- ResetN is pulled up in the Vitesse chip, but pulled down again by external 1k resistor.
                                                    -- So we need to assign a '1' to it.
    -- SO-DIMM Memory Bank I
    MB_I_IN                : IN    t_tech_ddr3_phy_in;     
    MB_I_IO                : INOUT t_tech_ddr3_phy_io; 
    MB_I_OU                : OUT   t_tech_ddr3_phy_ou 
  );
END apertif_unb1_fn_beamformer;


ARCHITECTURE str OF apertif_unb1_fn_beamformer IS

  CONSTANT c_use_transpose          : BOOLEAN := g_design_name="apertif_unb1_fn_beamformer_trans" OR g_design_name="apertif_unb1_fn_beamformer_tp_bg"; -- Also use DDR3
  CONSTANT c_use_bg                 : BOOLEAN := g_design_name="apertif_unb1_fn_beamformer_tp_bg"; -- Also use DDR3, but no mesh terminals
  CONSTANt c_use_bf                 : BOOLEAN := NOT(g_design_name="apertif_unb1_fn_beamformer_tp_bg" AND g_sim);
  CONSTANT c_use_phy                : t_c_unb1_board_use_phy  := (1, 1, sel_a_b(c_use_bg, 0, 1), 0, sel_a_b(c_use_transpose, 1, 0), 0, 0, 1); 
  CONSTANT c_fw_version             : t_unb1_board_fw_version := (3, 8);  -- firmware version x.y
  CONSTANT c_tr_mesh                : t_c_unb1_board_tr       := c_unb1_board_tr_mesh;
  CONSTANT c_dp_clk_use_pll         : BOOLEAN := g_design_name="apertif_unb1_fn_beamformer_base";

  -- System
  SIGNAL cs_sim                     : STD_LOGIC;
  SIGNAL xo_clk                     : STD_LOGIC;
  SIGNAL xo_rst                     : STD_LOGIC;
  SIGNAL xo_rst_n                   : STD_LOGIC;
  SIGNAL cal_clk                    : STD_LOGIC;
  SIGNAL cal_rec_clk                : STD_LOGIC;
  SIGNAL mm_clk                     : STD_LOGIC;
  SIGNAL mm_locked                  : STD_LOGIC;
  SIGNAL mm_rst                     : STD_LOGIC;
  SIGNAL dp_rst                     : STD_LOGIC;
  SIGNAL dp_clk                     : STD_LOGIC;
  SIGNAL dp_pps                     : STD_LOGIC;                             
  SIGNAL ddr_ref_rst                : STD_LOGIC;        
  SIGNAL sa_rst                     : STD_LOGIC;     
  SIGNAL ctrl_dp_rst                : STD_LOGIC;     
  SIGNAL ctrl_dp_clk                : STD_LOGIC;     

  SIGNAL this_chip_id               : STD_LOGIC_VECTOR(c_unb1_board_nof_chip_w-1 DOWNTO 0);  -- [2:0], so range 0-3 for FN and range 4-7 BN

  -- PIOs
  SIGNAL pout_debug_wave            : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL pout_wdi                   : STD_LOGIC;

  SIGNAL pin_intab                  : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0) := (OTHERS=>'0');
  SIGNAL pout_intab                 : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);

  -- eth1g UDP streaming ports
  SIGNAL eth1g_udp_tx_sosi_arr      : t_dp_sosi_arr(c_eth_nof_udp_ports-1 DOWNTO 0);
  SIGNAL eth1g_udp_tx_siso_arr      : t_dp_siso_arr(c_eth_nof_udp_ports-1 DOWNTO 0);  

  -- Interface: 10GbE
  CONSTANT c_nof_10GbE_offload_streams : NATURAL := 1;
  CONSTANT c_pkt_len                   : NATURAL := 176; -- Let tr_10GbE FIFO buffer one full packet before releasing it

  SIGNAL xaui_tx_arr                : t_xaui_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL xaui_rx_arr                : t_xaui_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL unb_xaui_tx_arr            : t_unb1_board_xaui_sl_2arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL unb_xaui_rx_arr            : t_unb1_board_xaui_sl_2arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL mdio_mdc_arr               : STD_LOGIC_VECTOR(c_nof_10GbE_offload_streams-1 DOWNTO 0);  
  SIGNAL mdio_mdat_in_arr           : STD_LOGIC_VECTOR(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL mdio_mdat_oen_arr          : STD_LOGIC_VECTOR(c_nof_10GbE_offload_streams-1 DOWNTO 0);

  SIGNAL beamlets_qua_sosi_arr      : t_dp_sosi_arr(g_bf.nof_bf_units-1 DOWNTO 0);  --  8b beamlets
  
  SIGNAL udp_offload_snk_in_arr     : t_dp_sosi_arr(g_bf.nof_bf_units-1 DOWNTO 0); 
  SIGNAL udp_offload_snk_out_arr    : t_dp_siso_arr(g_bf.nof_bf_units-1 DOWNTO 0) := (OTHERS => c_dp_siso_rdy); 

  ---------------------------------------------------- 
  -- Constants and signals for the reorder function -- 
  -------------------------------------------------- 
  CONSTANT c_offload_dat_w      : NATURAL := 8;
  
  CONSTANT c_use_complex        : BOOLEAN := TRUE;    
  CONSTANT c_total_data_w       : NATURAL := g_bf.nof_bf_units*c_offload_dat_w;
  CONSTANT c_complex_data_w     : NATURAL := c_total_data_w*c_nof_complex;
  CONSTANT c_data_w             : NATURAL := sel_a_b(c_use_complex, c_complex_data_w, c_total_data_w);
  
  CONSTANT c_wr_fifo_depth      : NATURAL := 256;
  CONSTANT c_rd_fifo_depth      : NATURAL := 256;
  CONSTANT c_ddr3_mem_w         : NATURAL := 256;
  CONSTANT c_ena_pre_transp     : BOOLEAN := TRUE;  
  CONSTANT c_wr_chunksize       : NATURAL := 176;  --240 for 6-bit or 176 for 8-bit; --g_bf.nof_weights;
  CONSTANT c_rd_chunksize       : NATURAL := 16;  --32;
  CONSTANT c_rd_nof_chunks      : NATURAL := 11;  --15 for 6-bit or 11 for 8-bit 
  CONSTANT c_rd_interval        : NATURAL := c_rd_chunksize;
  CONSTANT c_gapsize            : NATURAL := 0; 
  CONSTANT c_nof_blocks         : NATURAL := sel_a_b(g_sim, 16, 800000); --800000  781250);
  CONSTANT c_bsn_sync_time_out  : NATURAL := c_nof_blocks * g_bf.nof_weights;
  
  CONSTANT c_reorder_seq_conf   : t_reorder_seq := (c_wr_chunksize, 
                                                    c_rd_chunksize, 
                                                    c_rd_nof_chunks,  
                                                    c_rd_interval,
                                                    c_gapsize,      
                                                    c_nof_blocks);   
  
  CONSTANT c_tech_ddr           : t_c_tech_ddr := c_tech_ddr3_4g_800m_master;
  
  -- Signals to interface with the DDR conroller and memory model.
  SIGNAL ctlr_dvr_miso              : t_mem_ctlr_miso;   
  SIGNAL ctlr_dvr_mosi              : t_mem_ctlr_mosi;  
  
  SIGNAL to_mem_siso                : t_dp_siso := c_dp_siso_rdy;        
  SIGNAL to_mem_sosi                : t_dp_sosi;
  SIGNAL from_mem_siso              : t_dp_siso := c_dp_siso_rdy;
  SIGNAL from_mem_sosi              : t_dp_sosi;
    
  SIGNAL dp_offload_tx_src_out_arr  : t_dp_sosi_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL dp_offload_tx_src_in_arr   : t_dp_siso_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  
  SIGNAL bsn_in_sosi_arr            : t_dp_sosi_arr(2-1 DOWNTO 0);
  
  -----------------------------------------------------------------------------
  -- Memory Mapped interfaces
  -----------------------------------------------------------------------------  
  -- . WDI override
  SIGNAL reg_wdi_mosi               : t_mem_mosi;
  SIGNAL reg_wdi_miso               : t_mem_miso;
  -- . UniBoard system info
  SIGNAL reg_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso   : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso   : t_mem_miso;    
  -- . UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi          : t_mem_mosi;
  SIGNAL reg_unb_sens_miso          : t_mem_miso;   
  -- PPSH                                        
  SIGNAL reg_ppsh_mosi              : t_mem_mosi;  
  SIGNAL reg_ppsh_miso              : t_mem_miso;  
  -- . eth1g
  SIGNAL eth1g_tse_clk              : STD_LOGIC;
  SIGNAL eth1g_mm_rst               : STD_LOGIC;
  SIGNAL eth1g_tse_mosi             : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL eth1g_tse_miso             : t_mem_miso;
  SIGNAL eth1g_reg_mosi             : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL eth1g_reg_miso             : t_mem_miso;
  SIGNAL eth1g_reg_interrupt        : STD_LOGIC;
  SIGNAL eth1g_ram_mosi             : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL eth1g_ram_miso             : t_mem_miso;
  -- . tr_mesh
  SIGNAL tx_serial_2arr             : t_unb1_board_mesh_sl_2arr;
  SIGNAL rx_serial_2arr             : t_unb1_board_mesh_sl_2arr;    
  -- . tr_nonbonded with diagnostics
  SIGNAL reg_tr_nonbonded_mosi      : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_tr_nonbonded_miso      : t_mem_miso;
  SIGNAL reg_diagnostics_mosi       : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_diagnostics_miso       : t_mem_miso;
  -- . diag_data_buffer
  SIGNAL ram_diag_data_buf_mosi     : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_diag_data_buf_miso     : t_mem_miso;  
  SIGNAL reg_diag_data_buf_mosi     : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_diag_data_buf_miso     : t_mem_miso;  
  -- . bsn_monitor
  SIGNAL reg_bsn_monitor_mosi       : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_bsn_monitor_miso       : t_mem_miso;
  -- . block generator
  SIGNAL reg_diag_bg_mosi           : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_diag_bg_miso           : t_mem_miso;  
  SIGNAL ram_diag_bg_mosi           : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_diag_bg_miso           : t_mem_miso;               
  -- . beam former
  SIGNAL ram_ss_ss_wide_mosi        : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_ss_ss_wide_miso        : t_mem_miso;  
  SIGNAL ram_bf_weights_mosi        : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_bf_weights_miso        : t_mem_miso;                  
  SIGNAL ram_st_sst_bf_mosi         : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL ram_st_sst_bf_miso         : t_mem_miso;
  SIGNAL reg_st_sst_bf_mosi         : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_st_sst_bf_miso         : t_mem_miso;
  -- . dp_ram_from_mm for DP offload (header insertion)
  SIGNAL reg_dp_ram_from_mm_mosi    : t_mem_mosi;
  SIGNAL reg_dp_ram_from_mm_miso    : t_mem_miso := c_mem_miso_rst;
  SIGNAL ram_dp_ram_from_mm_mosi    : t_mem_mosi;
  SIGNAL ram_dp_ram_from_mm_miso    : t_mem_miso := c_mem_miso_rst;
  -- . UDP offload              
  SIGNAL reg_dp_split_mosi          : t_mem_mosi;
  SIGNAL reg_dp_split_miso          : t_mem_miso := c_mem_miso_rst;
  SIGNAL reg_dp_pkt_merge_mosi      : t_mem_mosi;
  SIGNAL reg_dp_pkt_merge_miso      : t_mem_miso := c_mem_miso_rst;
  -- . 10GbE offload
  SIGNAL reg_dp_offload_tx_mosi         : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_miso         : t_mem_miso;
  SIGNAL reg_dp_offload_tx_hdr_dat_mosi : t_mem_mosi;
  SIGNAL reg_dp_offload_tx_hdr_dat_miso : t_mem_miso;
  SIGNAL reg_tr_10GbE_mosi              : t_mem_mosi;
  SIGNAL reg_tr_10GbE_miso              : t_mem_miso := c_mem_miso_rst;
  SIGNAL reg_tr_xaui_mosi               : t_mem_mosi;
  SIGNAL reg_tr_xaui_miso               : t_mem_miso := c_mem_miso_rst;
  SIGNAL reg_mdio_mosi_arr              : t_mem_mosi_arr(c_unb1_board_nof_mdio-1 DOWNTO 0);
  SIGNAL reg_mdio_miso_arr              : t_mem_miso_arr(c_unb1_board_nof_mdio-1 DOWNTO 0);   
  -- . Pre-transpose
  SIGNAL ram_ss_ss_transp_mosi          : t_mem_mosi;
  SIGNAL ram_ss_ss_transp_miso          : t_mem_miso;
  -- . DDR register map                 
  SIGNAL reg_io_ddr_mosi                : t_mem_mosi;       
  SIGNAL reg_io_ddr_miso                : t_mem_miso;    
    -- . bsn_monitor
  SIGNAL reg_bsn_monitor_output_mosi    : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL reg_bsn_monitor_output_miso    : t_mem_miso;

  

BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb1_board_lib.ctrl_unb1_board
  GENERIC MAP (
    g_sim                     => g_sim,
    g_design_name             => g_design_name,
    g_design_note             => g_design_note,
    g_stamp_date              => g_stamp_date,
    g_stamp_time              => g_stamp_time, 
    g_stamp_svn               => g_stamp_svn, 
    g_fw_version              => c_fw_version,
    g_mm_clk_freq             => c_unb1_board_mm_clk_freq_50M,
    g_dp_clk_use_pll          => c_dp_clk_use_pll,    
    g_use_phy                 => c_use_phy,
    g_udp_offload             => TRUE, 
    g_aux                     => c_unb1_board_aux,
    g_udp_offload_nof_streams => c_eth_nof_udp_ports,  
    g_xo_clk_use_pll          => FALSE 
  )
  PORT MAP (
    -- Clock an reset signals
    cs_sim                   => cs_sim,
    xo_clk                   => xo_clk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk                   => mm_clk,
    mm_locked                => mm_locked,
    mm_rst                   => mm_rst,

    dp_rst                   => ctrl_dp_rst,
    dp_clk                   => ctrl_dp_clk,
    dp_pps                   => dp_pps,
    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,
    
    this_chip_id             => this_chip_id,
    
    -- Toggle WDI
    pout_debug_wave          => pout_debug_wave,
    pout_wdi                 => pout_wdi,

    -- MM buses
    -- . Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,
    
    -- . System_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 
    
    -- . UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
    
    -- . PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso,
    
    -- eth1g
    eth1g_tse_clk            => eth1g_tse_clk,   -- 125 MHz from xo_clk PLL in SOPC system
    eth1g_mm_rst             => eth1g_mm_rst,
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,

    -- eth1g UDP streaming ports to offload BF out
    udp_tx_sosi_arr        =>  eth1g_udp_tx_sosi_arr,
    udp_tx_siso_arr        =>  eth1g_udp_tx_siso_arr,
        
    -- FPGA pins
    -- . General
    CLK                      => CLK,
    PPS                      => PPS,
    WDI                      => WDI,
    INTA                     => INTA,
    INTB                     => INTB,
    -- . Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,
    -- . I2C Interface to Sensors
    sens_sc                  => sens_sc,
    sens_sd                  => sens_sd,        
    -- . 1GbE Control Interface
    ETH_clk                  => ETH_clk,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  -----------------------------------------------------------------------------
  -- Node function
  -----------------------------------------------------------------------------  
  u_node_fn_beamformer : ENTITY work.node_apertif_unb1_fn_beamformer
  GENERIC MAP(    
    g_sim           => g_sim,
    g_use_block_gen => c_use_bg, -- TRUE overrides terminal output and feeds BG output to the BF instead.
    g_use_bf        => c_use_bf 
  )
  PORT MAP(
    -- System
    mm_rst                      => mm_rst,
    mm_clk                      => mm_clk,
    dp_rst                      => dp_rst,
    dp_clk                      => dp_clk,
    dp_pps                      => dp_pps,
    tr_mesh_clk                 => SB_CLK,
    cal_clk                     => cal_rec_clk,

    chip_id                     => this_chip_id,

    -- MM interface
    -- . block generator
    reg_diag_bg_mosi            => reg_diag_bg_mosi,  
    reg_diag_bg_miso            => reg_diag_bg_miso,
    ram_diag_bg_mosi            => ram_diag_bg_mosi,  
    ram_diag_bg_miso            => ram_diag_bg_miso,  
    -- . beam former
    ram_ss_ss_wide_mosi         => ram_ss_ss_wide_mosi,  
    ram_ss_ss_wide_miso         => ram_ss_ss_wide_miso,  
    ram_bf_weights_mosi         => ram_bf_weights_mosi,   
    ram_bf_weights_miso         => ram_bf_weights_miso,   
    ram_st_sst_bf_mosi          => ram_st_sst_bf_mosi,      
    ram_st_sst_bf_miso          => ram_st_sst_bf_miso,
    reg_st_sst_bf_mosi          => reg_st_sst_bf_mosi,      
    reg_st_sst_bf_miso          => reg_st_sst_bf_miso,

    -- . hdr_insert for dp offload
    reg_hdr_insert_mosi         => reg_dp_ram_from_mm_mosi,
    ram_hdr_insert_mosi         => ram_dp_ram_from_mm_mosi,
    -- . dp_split for dp offload
    reg_dp_split_mosi           => reg_dp_split_mosi,
    reg_dp_split_miso           => reg_dp_split_miso,
    reg_dp_pkt_merge_mosi       => reg_dp_pkt_merge_mosi,
    reg_dp_pkt_merge_miso       => reg_dp_pkt_merge_miso,

    -- . tr_nonbonded
    reg_tr_nonbonded_mosi       => reg_tr_nonbonded_mosi,
    reg_tr_nonbonded_miso       => reg_tr_nonbonded_miso,
    reg_diagnostics_mosi        => reg_diagnostics_mosi,
    reg_diagnostics_miso        => reg_diagnostics_miso,
    -- . diag_data_buffer
    ram_diag_data_buf_mosi      => ram_diag_data_buf_mosi,
    ram_diag_data_buf_miso      => ram_diag_data_buf_miso,
    reg_diag_data_buf_mosi      => reg_diag_data_buf_mosi,
    reg_diag_data_buf_miso      => reg_diag_data_buf_miso,
    -- . bsn_monitor
    reg_bsn_monitor_mosi        => reg_bsn_monitor_mosi,
    reg_bsn_monitor_miso        => reg_bsn_monitor_miso,
   
    -- ST interface, BF beamlets out
    beamlets_bst_sosi_arr       => OPEN,                   -- 16b beamlets; for 1GbE a selection can be offloaded via bf_out_offload_tx_sosi_arr.
    beamlets_qua_sosi_arr       => beamlets_qua_sosi_arr,  --  8b beamlets; for 10GbE offload
    
    -- 1GbE offload 
    bf_out_offload_tx_sosi_arr  => eth1g_udp_tx_sosi_arr,
    bf_out_offload_tx_siso_arr  => eth1g_udp_tx_siso_arr,

    -- Mesh interface
    tx_serial_2arr              => tx_serial_2arr,
    rx_serial_2arr              => rx_serial_2arr
  );

  gen_no_transpose : IF c_use_transpose = FALSE GENERATE
    dp_rst <= ctrl_dp_rst;
    dp_clk <= ctrl_dp_clk;
    udp_offload_snk_in_arr <= beamlets_qua_sosi_arr; 
  END GENERATE;
                                                    
  gen_transpose : IF c_use_transpose = TRUE GENERATE
  ----------------------------------------------------------------------------
  -- Transpose function
  ---------------------------------------------------------------------------- 
    u_transpose: ENTITY reorder_lib.reorder_transpose
    GENERIC MAP(
      g_nof_streams      => g_bf.nof_bf_units,      
      g_in_dat_w         => c_offload_dat_w, 
      g_frame_size_in    => g_bf.nof_weights, -- 256              
      g_frame_size_out   => c_pkt_len,        -- 176
      g_use_complex      => c_use_complex, 
      g_mem_dat_w        => c_ddr3_mem_w,
      g_ena_pre_transp   => c_ena_pre_transp,                    
      g_reorder_seq      => c_reorder_seq_conf
    )                          
    PORT MAP (        
      mm_rst                => mm_rst, 
      mm_clk                => mm_clk,
    
      dp_rst                => dp_rst, 
      dp_clk                => dp_clk,
      
      -- ST sink                      
      snk_out_arr           => OPEN,
      snk_in_arr            => beamlets_qua_sosi_arr,
      
      -- ST source          
      src_in_arr            => udp_offload_snk_out_arr,
      src_out_arr           => udp_offload_snk_in_arr,
      
      ram_ss_ss_transp_mosi => ram_ss_ss_transp_mosi,
      ram_ss_ss_transp_miso => ram_ss_ss_transp_miso,
    
      -- Control interface to the external memory
      dvr_miso              => ctlr_dvr_miso,
      dvr_mosi              => ctlr_dvr_mosi,
      
      -- Data interface to the external memory
      to_mem_src_out        => to_mem_sosi,
      to_mem_src_in         => to_mem_siso,
      
      from_mem_snk_in       => from_mem_sosi,
      from_mem_snk_out      => from_mem_siso
    );
    
    u_areset_ddr_ref_rst : ENTITY common_lib.common_areset
    GENERIC MAP(
      g_rst_level => '1',
      g_delay_len => 40
    )
    PORT MAP(
      clk     => CLK,
      in_rst  => '0',
      out_rst => ddr_ref_rst
    );    
    
    u_ddr_mem_ctrl : ENTITY io_ddr_lib.io_ddr
    GENERIC MAP( 
      g_sim_model              => g_sim_model,
      g_technology             => c_tech_select_default, 
      g_tech_ddr               => c_tech_ddr,      
      g_cross_domain_dvr_ctlr  => FALSE, 
      g_wr_data_w              => c_data_w,        
      g_wr_fifo_depth          => c_wr_fifo_depth, 
      g_rd_fifo_depth          => c_rd_fifo_depth, 
      g_rd_data_w              => c_data_w,        
      g_wr_flush_mode          => "SYN",           
      g_wr_flush_use_channel   => FALSE,           
      g_wr_flush_start_channel => 0,               
      g_wr_flush_nof_channels  => 1                
    )                          
    PORT MAP ( 
      -- DDR reference clock
      ctlr_ref_clk    => CLK,
      ctlr_ref_rst    => ddr_ref_rst,
    
      -- DDR controller clock domain
      ctlr_clk_out    => dp_clk, 
      ctlr_rst_out    => dp_rst,   
                      
      ctlr_clk_in     => dp_clk,  
      ctlr_rst_in     => dp_rst,  

      -- MM clock + reset
      mm_rst          => mm_rst,
      mm_clk          => mm_clk,     
      
      -- MM register map for DDR controller status info
      reg_io_ddr_mosi => reg_io_ddr_mosi,
      reg_io_ddr_miso => reg_io_ddr_miso,
   
      -- Driver clock domain
      dvr_clk         => dp_clk,
      dvr_rst         => dp_rst,
                      
      dvr_miso        => ctlr_dvr_miso,
      dvr_mosi        => ctlr_dvr_mosi,
    
      -- Write FIFO clock domain
      wr_clk          => dp_clk,
      wr_rst          => dp_rst,
    
      wr_fifo_usedw   => OPEN,
      wr_sosi         => to_mem_sosi,  
      wr_siso         => to_mem_siso,
    
      -- Read FIFO clock domain
      rd_clk          => dp_clk,
      rd_rst          => dp_rst,     
      
      rd_fifo_usedw   => OPEN,
      rd_sosi         => from_mem_sosi,
      rd_siso         => from_mem_siso,     
                      
      phy3_in         => MB_I_IN,
      phy3_io         => MB_I_IO,  
      phy3_ou         => MB_I_OU
    );
  END GENERATE;
  
  -----------------------------------------------------------------------------
  -- Mesh I/O
  -----------------------------------------------------------------------------  
  no_tr_mesh : IF c_use_phy.tr_mesh=0 GENERATE
    rx_serial_2arr <= (OTHERS=>(OTHERS=>'0'));
  END GENERATE;
  
  gen_tr_mesh : IF c_use_phy.tr_mesh/=0 GENERATE
    u_mesh_io : ENTITY unb1_board_lib.unb1_board_mesh_io
    GENERIC MAP (
      g_bus_w => c_tr_mesh.bus_w
    )
    PORT MAP (
      tx_serial_2arr => tx_serial_2arr,
      rx_serial_2arr => rx_serial_2arr,
      
      -- Serial I/O
      FN_BN_0_TX     => FN_BN_0_TX,
      FN_BN_0_RX     => FN_BN_0_RX,
      FN_BN_1_TX     => FN_BN_1_TX,
      FN_BN_1_RX     => FN_BN_1_RX,
      FN_BN_2_TX     => FN_BN_2_TX,
      FN_BN_2_RX     => FN_BN_2_RX,
      FN_BN_3_TX     => FN_BN_3_TX,
      FN_BN_3_RX     => FN_BN_3_RX
    );
  END GENERATE;

  -----------------------------------------------------------------------------
  -- Interface : 10GbE
  -----------------------------------------------------------------------------
  u_areset_sa_rst : ENTITY common_lib.common_areset
  GENERIC MAP(
    g_rst_level => '1',
    g_delay_len => 4
  )
  PORT MAP(
    clk     => SA_CLK,
    in_rst  => '0',
    out_rst => sa_rst
  );    
  
  u_tr_10GbE: ENTITY tr_10GbE_lib.tr_10GbE
  GENERIC MAP(
    g_sim             => g_sim,
    g_sim_level       => 0,
    g_nof_macs        => c_nof_10GbE_offload_streams,
    g_use_mdio        => TRUE,
    g_tx_fifo_fill    => c_pkt_len, 
    g_tx_fifo_size    => 2*c_pkt_len,
    g_rx_fifo_size    => 2*c_pkt_len
  )                      
  
  PORT MAP (  
    -- Transceiver PLL reference clock
    tr_ref_clk_156    => SA_CLK, 
    tr_ref_rst_156    => sa_rst,

    -- Calibration & reconfig clock
    cal_rec_clk       => cal_rec_clk, --cal_clk, # required for XAUI
    
    -- MM interface       
    mm_rst            => mm_rst,  
    mm_clk            => mm_clk,

    reg_mac_mosi      => reg_tr_10GbE_mosi,
    reg_mac_miso      => reg_tr_10GbE_miso,

    xaui_mosi         => reg_tr_xaui_mosi,
    xaui_miso         => reg_tr_xaui_miso,

    mdio_mosi_arr     => reg_mdio_mosi_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0),
    mdio_miso_arr     => reg_mdio_miso_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0),

    -- DP interface
    dp_rst            => dp_rst,
    dp_clk            => dp_clk,

    snk_out_arr       => dp_offload_tx_src_in_arr,
    snk_in_arr        => dp_offload_tx_src_out_arr,

    -- Serial XAUI IO
    xaui_tx_arr       => xaui_tx_arr, 
    xaui_rx_arr       => xaui_rx_arr, 
  
    -- MDIO interface
    mdio_rst          => SI_FN_RSTN,
    mdio_mdc_arr      => mdio_mdc_arr,
    mdio_mdat_in_arr  => mdio_mdat_in_arr,
    mdio_mdat_oen_arr => mdio_mdat_oen_arr
  );

  -- Wire together different types
  gen_wires: FOR i IN 0 TO c_nof_10GbE_offload_streams-1 GENERATE
    unb_xaui_tx_arr(i) <= xaui_tx_arr(i);
    xaui_rx_arr(i)     <= unb_xaui_rx_arr(i);
  END GENERATE;

  u_front_io : ENTITY unb1_board_lib.unb1_board_front_io
  GENERIC MAP (
    g_nof_xaui => c_nof_10GbE_offload_streams
  )
  PORT MAP (
    xaui_tx_arr       => unb_xaui_tx_arr,
    xaui_rx_arr       => unb_xaui_rx_arr,
   
    mdio_mdc_arr      => mdio_mdc_arr,
    mdio_mdat_in_arr  => mdio_mdat_in_arr,
    mdio_mdat_oen_arr => mdio_mdat_oen_arr,

    -- Serial I/O
    SI_FN_0_TX        => SI_FN_0_TX,
    SI_FN_0_RX        => SI_FN_0_RX,
    SI_FN_1_TX        => SI_FN_1_TX,
    SI_FN_1_RX        => SI_FN_1_RX,
    SI_FN_2_TX        => SI_FN_2_TX,
    SI_FN_2_RX        => SI_FN_2_RX,

    SI_FN_0_CNTRL     => SI_FN_0_CNTRL,
    SI_FN_1_CNTRL     => SI_FN_1_CNTRL,
    SI_FN_2_CNTRL     => SI_FN_2_CNTRL,
    SI_FN_3_CNTRL     => SI_FN_3_CNTRL
  );
  
  u_bsn_monitor_align : ENTITY dp_lib.mms_dp_bsn_monitor
  GENERIC MAP (
    g_nof_streams        => 2, 
    g_cross_clock_domain => TRUE,    
    g_sync_timeout       => c_bsn_sync_time_out,
    g_bsn_w              => c_dp_stream_bsn_w,
    g_cnt_sop_w          => c_word_w,
    g_cnt_valid_w        => c_word_w,
    g_log_first_bsn      => TRUE
  )
  PORT MAP (
    -- Memory-mapped clock domain
    mm_rst      => mm_rst,
    mm_clk      => mm_clk,
    reg_mosi    => reg_bsn_monitor_output_mosi,
    reg_miso    => reg_bsn_monitor_output_miso,
    
    -- Streaming clock domain
    dp_rst      => dp_rst,
    dp_clk      => dp_clk,
    in_siso_arr => (OTHERS => c_dp_siso_rdy),
    in_sosi_arr => bsn_in_sosi_arr(1 DOWNTO 0)
  );
  bsn_in_sosi_arr(0) <= udp_offload_snk_in_arr(0);
  bsn_in_sosi_arr(1) <= dp_offload_tx_src_out_arr(0);
  
  -----------------------------------------------------------------------------
  -- DP offload TX : BF out -> 10GbE
  -----------------------------------------------------------------------------  
  u_fn_beamformer_udp_offload : ENTITY work.apertif_unb1_fn_beamformer_udp_offload
  PORT MAP (
    mm_rst                         => mm_rst,
    mm_clk                         => mm_clk,
    
    dp_rst                         => dp_rst,
    dp_clk                         => dp_clk,

    snk_in_arr                     => udp_offload_snk_in_arr,  -- 8b beamlets

    src_out_arr                    => dp_offload_tx_src_out_arr,
    src_in_arr                     => dp_offload_tx_src_in_arr,

    ID                             => ID,

    reg_dp_offload_tx_mosi         => reg_dp_offload_tx_mosi,
    reg_dp_offload_tx_miso         => reg_dp_offload_tx_miso,
    reg_dp_offload_tx_hdr_dat_mosi => reg_dp_offload_tx_hdr_dat_mosi,
    reg_dp_offload_tx_hdr_dat_miso => reg_dp_offload_tx_hdr_dat_miso
  );
 
  u_mmm : ENTITY work.mmm_apertif_unb1_fn_beamformer 
  GENERIC MAP(
    -- General
    g_sim         => g_sim,         
    g_sim_unb_nr  => g_sim_unb_nr,  
    g_sim_node_nr => g_sim_node_nr, 
    g_bf          => g_bf,
    g_reorder_seq => c_reorder_seq_conf         
  )
  PORT MAP (
    xo_clk         => xo_clk,
    xo_rst_n       => xo_rst_n,                    
    xo_rst         => xo_rst,                      

    mm_rst         => mm_rst,                      
    mm_clk         => mm_clk,                      
    mm_locked      => mm_locked,                   
    cal_clk        => cal_rec_clk,                     

    pout_wdi       => pout_wdi,                    
    
    -- Manual WDI override
    reg_wdi_mosi               => reg_wdi_mosi,                  
    reg_wdi_miso               => reg_wdi_miso,                  

    -- system_info             
    reg_unb_system_info_mosi   => reg_unb_system_info_mosi,      
    reg_unb_system_info_miso   => reg_unb_system_info_miso,      
    rom_unb_system_info_mosi   => rom_unb_system_info_mosi,      
    rom_unb_system_info_miso   => rom_unb_system_info_miso,      

    -- UniBoard I2C sensors    
    reg_unb_sens_mosi          => reg_unb_sens_mosi,              
    reg_unb_sens_miso          => reg_unb_sens_miso,   
    
    -- PPSH
    reg_ppsh_mosi              => reg_ppsh_mosi,
    reg_ppsh_miso              => reg_ppsh_miso, 

    -- Diagnostics             
    reg_diagnostics_mosi       => reg_diagnostics_mosi,           
    reg_diagnostics_miso       => reg_diagnostics_miso,          

    -- . tr_nonbonded with diagnostics
    reg_tr_nonbonded_mosi      => reg_tr_nonbonded_mosi, 
    reg_tr_nonbonded_miso      => reg_tr_nonbonded_miso, 

    -- . diag_data_buffer      
    ram_diag_data_buf_mosi     => ram_diag_data_buf_mosi,
    ram_diag_data_buf_miso     => ram_diag_data_buf_miso,
    reg_diag_data_buf_mosi     => reg_diag_data_buf_mosi,
    reg_diag_data_buf_miso     => reg_diag_data_buf_miso,

    -- . bsn_monitor           
    reg_bsn_monitor_mosi       => reg_bsn_monitor_mosi,  
    reg_bsn_monitor_miso       => reg_bsn_monitor_miso,  

    -- . beamformer            
    ram_st_sst_bf_mosi         => ram_st_sst_bf_mosi,    
    ram_st_sst_bf_miso         => ram_st_sst_bf_miso,    
    reg_st_sst_bf_mosi         => reg_st_sst_bf_mosi,    
    reg_st_sst_bf_miso         => reg_st_sst_bf_miso,    

    ram_ss_ss_wide_mosi        => ram_ss_ss_wide_mosi,   
    ram_ss_ss_wide_miso        => ram_ss_ss_wide_miso,   
    ram_bf_weights_mosi        => ram_bf_weights_mosi,   
    ram_bf_weights_miso        => ram_bf_weights_miso,   

    reg_diag_bg_mosi           => reg_diag_bg_mosi,      
    reg_diag_bg_miso           => reg_diag_bg_miso,      
    ram_diag_bg_mosi           => ram_diag_bg_mosi,      
    ram_diag_bg_miso           => ram_diag_bg_miso,      

    -- . dp_ram_from_mm for DP offload (header insertion)
    reg_dp_ram_from_mm_mosi        => reg_dp_ram_from_mm_mosi,        
    reg_dp_ram_from_mm_miso        => reg_dp_ram_from_mm_miso,        
    ram_dp_ram_from_mm_mosi        => ram_dp_ram_from_mm_mosi,        
    ram_dp_ram_from_mm_miso        => ram_dp_ram_from_mm_miso,        
                       
    -- . UDP offload   
    reg_dp_split_mosi              => reg_dp_split_mosi,              
    reg_dp_split_miso              => reg_dp_split_miso,              
    reg_dp_pkt_merge_mosi          => reg_dp_pkt_merge_mosi,          
    reg_dp_pkt_merge_miso          => reg_dp_pkt_merge_miso,          
                       
    -- . 10GbE offload 
    reg_dp_offload_tx_mosi         => reg_dp_offload_tx_mosi,         
    reg_dp_offload_tx_miso         => reg_dp_offload_tx_miso,         
    reg_dp_offload_tx_hdr_dat_mosi => reg_dp_offload_tx_hdr_dat_mosi, 
    reg_dp_offload_tx_hdr_dat_miso => reg_dp_offload_tx_hdr_dat_miso, 

    reg_tr_10GbE_mosi              => reg_tr_10GbE_mosi,              
    reg_tr_10GbE_miso              => reg_tr_10GbE_miso,              

    reg_mdio_mosi_arr              => reg_mdio_mosi_arr,              
    reg_mdio_miso_arr              => reg_mdio_miso_arr,              

    reg_tr_xaui_mosi               => reg_tr_xaui_mosi,               
    reg_tr_xaui_miso               => reg_tr_xaui_miso,    
    
    -- . Pre-transpose
    ram_ss_ss_transp_mosi          => ram_ss_ss_transp_mosi, 
    ram_ss_ss_transp_miso          => ram_ss_ss_transp_miso,   
    
    -- . DDR register map 
    reg_io_ddr_mosi                => reg_io_ddr_mosi,
    reg_io_ddr_miso                => reg_io_ddr_miso,   
    
    -- . bsn_monitor_output                                                 
    reg_bsn_monitor_output_mosi    => reg_bsn_monitor_output_mosi,
    reg_bsn_monitor_output_miso    => reg_bsn_monitor_output_miso,
    
    -- eth1g     
    eth1g_tse_clk                  => eth1g_tse_clk,                  
    eth1g_mm_rst                   => eth1g_mm_rst,                   
    eth1g_tse_mosi                 => eth1g_tse_mosi,                 
    eth1g_tse_miso                 => eth1g_tse_miso,                 
    eth1g_reg_mosi                 => eth1g_reg_mosi,                 
    eth1g_reg_miso                 => eth1g_reg_miso,                 
    eth1g_reg_interrupt            => eth1g_reg_interrupt,
    eth1g_ram_mosi                 => eth1g_ram_mosi,                 
    eth1g_ram_miso                 => eth1g_ram_miso                 
  );
END;













