------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------
LIBRARY IEEE, common_lib, unb1_board_lib, dp_lib, technology_lib, io_ddr_lib, eth_lib, tech_tse_lib, bf_lib, tech_ddr_lib, tr_10GbE_lib, reorder_lib, diag_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE tech_ddr_lib.tech_ddr_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE tech_tse_lib.tech_tse_pkg.ALL;
USE tech_tse_lib.tb_tech_tse_pkg.ALL;  
USE reorder_lib.reorder_pkg.ALL;
USE bf_lib.bf_pkg.ALL;
USE diag_lib.diag_pkg.ALL;

ENTITY apertif_unb1_fn_bf_emu IS
  GENERIC (
    g_design_name : STRING  := "apertif_unb1_fn_bf_emu"; --"apertif_unb1_fn_bf_emu";
    g_design_note : STRING  := "revision info"; --"UNUSED";
    g_sim         : BOOLEAN := FALSE; --Overridden by TB
    g_sim_model   : BOOLEAN := FALSE; -- Use fast DDR3 model
    g_sim_unb_nr  : NATURAL := 0;
    g_sim_node_nr : NATURAL := 0;
    g_stamp_date  : NATURAL := 0;  -- Date (YYYYMMDD) -- set by QSF
    g_stamp_time  : NATURAL := 0;  -- Time (HHMMSS)   -- set by QSF
    g_stamp_svn   : NATURAL := 0;  -- SVN revision    -- set by QSF
    g_bf          : t_c_bf  := c_bf
  );
  PORT (
   -- GENERAL
    CLK                    : IN    STD_LOGIC; -- System Clock
    PPS                    : IN    STD_LOGIC; -- System Sync
    WDI                    : OUT   STD_LOGIC; -- Watchdog Clear
    INTA                   : INOUT STD_LOGIC; -- FPGA interconnect line
    INTB                   : INOUT STD_LOGIC; -- FPGA interconnect line

    -- Others
    VERSION                : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.version_w-1 DOWNTO 0);
    ID                     : IN    STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);
    TESTIO                 : INOUT STD_LOGIC_VECTOR(c_unb1_board_aux.testio_w-1 DOWNTO 0);

    -- I2C Interface to Sensors
    sens_sc                : INOUT STD_LOGIC;
    sens_sd                : INOUT STD_LOGIC;

    -- 1GbE Control Interface
    ETH_clk                : IN    STD_LOGIC;
    ETH_SGIN               : IN    STD_LOGIC;
    ETH_SGOUT              : OUT   STD_LOGIC;
    
    -- Transceiver clocks
    SA_CLK                 : IN  STD_LOGIC := '0';  -- TR clock BN-BI (tr_back) / SI_FN (tr_front)

    -- Front Serial I/O
    SI_FN_0_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_0_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_1_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_2_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_TX             : OUT   STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);
    SI_FN_3_RX             : IN    STD_LOGIC_VECTOR(c_unb1_board_ci.tr.bus_w-1 DOWNTO 0);

    SI_FN_0_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0); -- (0 = LASI; 1=MDC; 2=MDIO)
    SI_FN_1_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_2_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_3_CNTRL          : INOUT STD_LOGIC_VECTOR(c_unb1_board_ci.tr.cntrl_w-1 DOWNTO 0);
    SI_FN_RSTN             : OUT   STD_LOGIC := '1' -- ResetN is pulled up in the Vitesse chip, but pulled down again by external 1k resistor.
                                                    -- So we need to assign a '1' to it.
  );
END apertif_unb1_fn_bf_emu;


ARCHITECTURE str OF apertif_unb1_fn_bf_emu IS

  CONSTANT c_use_phy                : t_c_unb1_board_use_phy  := (1, 1, 0, 0, 0, 0, 0, 1); 
  CONSTANT c_fw_version             : t_unb1_board_fw_version := (0, 0);  -- firmware version x.y
  CONSTANT c_dp_clk_use_pll         : BOOLEAN := TRUE;

  -- System
  SIGNAL cs_sim                     : STD_LOGIC;
  SIGNAL xo_clk                     : STD_LOGIC;
  SIGNAL xo_rst                     : STD_LOGIC;
  SIGNAL xo_rst_n                   : STD_LOGIC;
  SIGNAL cal_clk                    : STD_LOGIC;
  SIGNAL cal_rec_clk                : STD_LOGIC;
  SIGNAL mm_clk                     : STD_LOGIC;
  SIGNAL mm_locked                  : STD_LOGIC;
  SIGNAL mm_rst                     : STD_LOGIC;
  SIGNAL dp_rst                     : STD_LOGIC;
  SIGNAL dp_clk                     : STD_LOGIC;
  SIGNAL dp_pps                     : STD_LOGIC;                             
  SIGNAL ddr_ref_rst                : STD_LOGIC;        
  SIGNAL sa_rst                     : STD_LOGIC;     
  SIGNAL ctrl_dp_rst                : STD_LOGIC;     
  SIGNAL ctrl_dp_clk                : STD_LOGIC;     

  SIGNAL this_chip_id               : STD_LOGIC_VECTOR(c_unb1_board_nof_chip_w-1 DOWNTO 0);  -- [2:0], so range 0-3 for FN and range 4-7 BN

  -- PIOs
  SIGNAL pout_debug_wave            : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);
  SIGNAL pout_wdi                   : STD_LOGIC;

  -- Block generator
  CONSTANT c_bg_block_size          : NATURAL := 256;
  CONSTANT c_bg_gapsize             : NATURAL := 0;
  CONSTANT c_bg_blocks_per_sync     : NATURAL := 781250;
  CONSTANT c_bg_ctrl                : t_diag_block_gen := (sel_a_b(g_sim, '1', '0'), -- enable: On by default in simulation; MM enable required on hardware.
                                                           '0',                      -- enable_sync        
                                                          TO_UVEC(     c_bg_block_size, c_diag_bg_samples_per_packet_w),
                                                          TO_UVEC(c_bg_blocks_per_sync, c_diag_bg_blocks_per_sync_w),
                                                          TO_UVEC(        c_bg_gapsize, c_diag_bg_gapsize_w),
                                                          TO_UVEC(                   0, c_diag_bg_mem_low_adrs_w),
                                                          TO_UVEC(   c_bg_block_size-1, c_diag_bg_mem_high_adrs_w),
                                                          TO_UVEC(                   0, c_diag_bg_bsn_init_w));
  -- Interface: 10GbE
  CONSTANT c_nof_10GbE_offload_streams : NATURAL := 1;
  CONSTANT c_pkt_len                   : NATURAL := 176; -- Let tr_10GbE FIFO buffer one full packet before releasing it

  SIGNAL xaui_tx_arr                : t_xaui_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL xaui_rx_arr                : t_xaui_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL unb_xaui_tx_arr            : t_unb1_board_xaui_sl_2arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL unb_xaui_rx_arr            : t_unb1_board_xaui_sl_2arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL mdio_mdc_arr               : STD_LOGIC_VECTOR(c_nof_10GbE_offload_streams-1 DOWNTO 0);  
  SIGNAL mdio_mdat_in_arr           : STD_LOGIC_VECTOR(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL mdio_mdat_oen_arr          : STD_LOGIC_VECTOR(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  
  SIGNAL dp_offload_tx_snk_in_arr   : t_dp_sosi_arr(g_bf.nof_bf_units-1 DOWNTO 0); 
  SIGNAL dp_offload_tx_snk_out_arr  : t_dp_siso_arr(g_bf.nof_bf_units-1 DOWNTO 0) := (OTHERS => c_dp_siso_rdy); 
      
  SIGNAL dp_offload_tx_src_out_arr  : t_dp_sosi_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
  SIGNAL dp_offload_tx_src_in_arr   : t_dp_siso_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0);
    
  -----------------------------------------------------------------------------
  -- Memory Mapped interfaces
  -----------------------------------------------------------------------------  
  -- . WDI override
  SIGNAL reg_wdi_mosi               : t_mem_mosi;
  SIGNAL reg_wdi_miso               : t_mem_miso;
  -- . UniBoard system info
  SIGNAL reg_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL reg_unb_system_info_miso   : t_mem_miso;
  SIGNAL rom_unb_system_info_mosi   : t_mem_mosi;
  SIGNAL rom_unb_system_info_miso   : t_mem_miso;    
  -- . UniBoard I2C sens
  SIGNAL reg_unb_sens_mosi          : t_mem_mosi;
  SIGNAL reg_unb_sens_miso          : t_mem_miso;   
  -- PPSH                                        
  SIGNAL reg_ppsh_mosi              : t_mem_mosi;  
  SIGNAL reg_ppsh_miso              : t_mem_miso;  
  -- Block gens                                    
  SIGNAL reg_diag_bg_mosi              : t_mem_mosi;  
  SIGNAL reg_diag_bg_miso              : t_mem_miso;  
  SIGNAL ram_diag_bg_mosi              : t_mem_mosi;  
  SIGNAL ram_diag_bg_miso              : t_mem_miso;
  -- . eth1g
  SIGNAL eth1g_tse_clk              : STD_LOGIC;
  SIGNAL eth1g_mm_rst               : STD_LOGIC;
  SIGNAL eth1g_tse_mosi             : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL eth1g_tse_miso             : t_mem_miso;
  SIGNAL eth1g_reg_mosi             : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL eth1g_reg_miso             : t_mem_miso;
  SIGNAL eth1g_reg_interrupt        : STD_LOGIC;
  SIGNAL eth1g_ram_mosi             : t_mem_mosi := c_mem_mosi_rst;
  SIGNAL eth1g_ram_miso             : t_mem_miso;           
  -- . 10GbE offload
  SIGNAL reg_tr_10GbE_mosi              : t_mem_mosi;
  SIGNAL reg_tr_10GbE_miso              : t_mem_miso := c_mem_miso_rst;
  SIGNAL reg_tr_xaui_mosi               : t_mem_mosi;
  SIGNAL reg_tr_xaui_miso               : t_mem_miso := c_mem_miso_rst;
  SIGNAL reg_mdio_mosi_arr              : t_mem_mosi_arr(c_unb1_board_nof_mdio-1 DOWNTO 0);
  SIGNAL reg_mdio_miso_arr              : t_mem_miso_arr(c_unb1_board_nof_mdio-1 DOWNTO 0);     

BEGIN

  -----------------------------------------------------------------------------
  -- General control function
  -----------------------------------------------------------------------------
  u_ctrl : ENTITY unb1_board_lib.ctrl_unb1_board
  GENERIC MAP (
    g_sim                     => g_sim,
    g_design_name             => g_design_name,
    g_design_note             => g_design_note,
    g_stamp_date              => g_stamp_date,
    g_stamp_time              => g_stamp_time, 
    g_stamp_svn               => g_stamp_svn, 
    g_fw_version              => c_fw_version,
    g_mm_clk_freq             => c_unb1_board_mm_clk_freq_50M,
    g_dp_clk_use_pll          => c_dp_clk_use_pll,    
    g_use_phy                 => c_use_phy,
    g_udp_offload             => TRUE, 
    g_aux                     => c_unb1_board_aux,
    g_udp_offload_nof_streams => c_eth_nof_udp_ports,  
    g_xo_clk_use_pll          => FALSE 
  )
  PORT MAP (
    -- Clock an reset signals
    cs_sim                   => cs_sim,
    xo_clk                   => xo_clk,
    xo_rst                   => xo_rst,
    xo_rst_n                 => xo_rst_n,

    mm_clk                   => mm_clk,
    mm_locked                => mm_locked,
    mm_rst                   => mm_rst,

    dp_rst                   => ctrl_dp_rst,
    dp_clk                   => ctrl_dp_clk,
    dp_pps                   => dp_pps,
    dp_rst_in                => dp_rst,
    dp_clk_in                => dp_clk,
    
    this_chip_id             => this_chip_id,
    
    -- Toggle WDI
    pout_debug_wave          => pout_debug_wave,
    pout_wdi                 => pout_wdi,

    -- MM buses
    -- . Manual WDI override
    reg_wdi_mosi             => reg_wdi_mosi,
    reg_wdi_miso             => reg_wdi_miso,
    
    -- . System_info
    reg_unb_system_info_mosi => reg_unb_system_info_mosi,
    reg_unb_system_info_miso => reg_unb_system_info_miso, 
    rom_unb_system_info_mosi => rom_unb_system_info_mosi,
    rom_unb_system_info_miso => rom_unb_system_info_miso, 
    
    -- . UniBoard I2C sensors
    reg_unb_sens_mosi        => reg_unb_sens_mosi,
    reg_unb_sens_miso        => reg_unb_sens_miso,    
    
    -- . PPSH
    reg_ppsh_mosi            => reg_ppsh_mosi,
    reg_ppsh_miso            => reg_ppsh_miso,
    
    -- eth1g
    eth1g_tse_clk            => eth1g_tse_clk,   -- 125 MHz from xo_clk PLL in SOPC system
    eth1g_mm_rst             => eth1g_mm_rst,
    eth1g_tse_mosi           => eth1g_tse_mosi,
    eth1g_tse_miso           => eth1g_tse_miso,
    eth1g_reg_mosi           => eth1g_reg_mosi,
    eth1g_reg_miso           => eth1g_reg_miso,
    eth1g_reg_interrupt      => eth1g_reg_interrupt,
    eth1g_ram_mosi           => eth1g_ram_mosi,
    eth1g_ram_miso           => eth1g_ram_miso,

    -- FPGA pins
    -- . General
    CLK                      => CLK,
    PPS                      => PPS,
    WDI                      => WDI,
    INTA                     => INTA,
    INTB                     => INTB,
    -- . Others
    VERSION                  => VERSION,
    ID                       => ID,
    TESTIO                   => TESTIO,
    -- . I2C Interface to Sensors
    sens_sc                  => sens_sc,
    sens_sd                  => sens_sd,        
    -- . 1GbE Control Interface
    ETH_clk                  => ETH_clk,
    ETH_SGIN                 => ETH_SGIN,
    ETH_SGOUT                => ETH_SGOUT
  );

  -----------------------------------------------------------------------------
  -- Interface : 10GbE
  -----------------------------------------------------------------------------
  u_areset_sa_rst : ENTITY common_lib.common_areset
  GENERIC MAP(
    g_rst_level => '1',
    g_delay_len => 4
  )
  PORT MAP(
    clk     => SA_CLK,
    in_rst  => '0',
    out_rst => sa_rst
  );    
  
  u_tr_10GbE: ENTITY tr_10GbE_lib.tr_10GbE
  GENERIC MAP(
    g_sim             => g_sim,
    g_sim_level       => 0,
    g_nof_macs        => c_nof_10GbE_offload_streams,
    g_use_mdio        => TRUE,
    g_tx_fifo_fill    => c_pkt_len, 
    g_tx_fifo_size    => 2*c_pkt_len,
    g_rx_fifo_size    => 2*c_pkt_len
  )                      
  
  PORT MAP (  
    -- Transceiver PLL reference clock
    tr_ref_clk_156    => SA_CLK, 
    tr_ref_rst_156    => sa_rst,

    -- Calibration & reconfig clock
    cal_rec_clk       => cal_rec_clk, --cal_clk, # required for XAUI
    
    -- MM interface       
    mm_rst            => mm_rst,  
    mm_clk            => mm_clk,

    reg_mac_mosi      => reg_tr_10GbE_mosi,
    reg_mac_miso      => reg_tr_10GbE_miso,

    xaui_mosi         => reg_tr_xaui_mosi,
    xaui_miso         => reg_tr_xaui_miso,

    mdio_mosi_arr     => reg_mdio_mosi_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0),
    mdio_miso_arr     => reg_mdio_miso_arr(c_nof_10GbE_offload_streams-1 DOWNTO 0),

    -- DP interface
    dp_rst            => dp_rst,
    dp_clk            => dp_clk,

    snk_out_arr       => dp_offload_tx_src_in_arr,
    snk_in_arr        => dp_offload_tx_src_out_arr,

    -- Serial XAUI IO
    xaui_tx_arr       => xaui_tx_arr, 
    xaui_rx_arr       => xaui_rx_arr, 
  
    -- MDIO interface
    mdio_rst          => SI_FN_RSTN,
    mdio_mdc_arr      => mdio_mdc_arr,
    mdio_mdat_in_arr  => mdio_mdat_in_arr,
    mdio_mdat_oen_arr => mdio_mdat_oen_arr
  );

  -- Wire together different types
  gen_wires: FOR i IN 0 TO c_nof_10GbE_offload_streams-1 GENERATE
    unb_xaui_tx_arr(i) <= xaui_tx_arr(i);
    xaui_rx_arr(i)     <= unb_xaui_rx_arr(i);
  END GENERATE;

  u_front_io : ENTITY unb1_board_lib.unb1_board_front_io
  GENERIC MAP (
    g_nof_xaui => c_nof_10GbE_offload_streams
  )
  PORT MAP (
    xaui_tx_arr       => unb_xaui_tx_arr,
    xaui_rx_arr       => unb_xaui_rx_arr,
   
    mdio_mdc_arr      => mdio_mdc_arr,
    mdio_mdat_in_arr  => mdio_mdat_in_arr,
    mdio_mdat_oen_arr => mdio_mdat_oen_arr,

    -- Serial I/O
    SI_FN_0_TX        => SI_FN_0_TX,
    SI_FN_0_RX        => SI_FN_0_RX,
    SI_FN_1_TX        => SI_FN_1_TX,
    SI_FN_1_RX        => SI_FN_1_RX,
    SI_FN_2_TX        => SI_FN_2_TX,
    SI_FN_2_RX        => SI_FN_2_RX,

    SI_FN_0_CNTRL     => SI_FN_0_CNTRL,
    SI_FN_1_CNTRL     => SI_FN_1_CNTRL,
    SI_FN_2_CNTRL     => SI_FN_2_CNTRL,
    SI_FN_3_CNTRL     => SI_FN_3_CNTRL
  );

  -----------------------------------------------------------------------------
  -- Block generators: one for every BF unit (4)
  -----------------------------------------------------------------------------
  u_mms_diag_block_gen : ENTITY diag_lib.mms_diag_block_gen
  GENERIC MAP (
    g_nof_streams        => 4,
    g_buf_dat_w          => 16,
    g_buf_addr_w         => 256,
    g_file_name_prefix   => "hex/bg_data",
    g_diag_block_gen_rst => c_bg_ctrl
  )
  PORT MAP (
    mm_rst           => mm_rst,
    mm_clk           => mm_clk,
  
    dp_rst           => dp_rst,
    dp_clk           => dp_clk,

    reg_bg_ctrl_mosi => reg_diag_bg_mosi,
    reg_bg_ctrl_miso => reg_diag_bg_miso,
  
    out_sosi_arr     => dp_offload_tx_snk_in_arr
  );
    
  -----------------------------------------------------------------------------
  -- DP offload TX : BF out -> 10GbE
  -----------------------------------------------------------------------------  
  u_fn_beamformer_udp_offload : ENTITY work.apertif_unb1_fn_beamformer_udp_offload
  PORT MAP (
    mm_rst                         => mm_rst,
    mm_clk                         => mm_clk,
    
    dp_rst                         => dp_rst,
    dp_clk                         => dp_clk,

    snk_in_arr                     => dp_offload_tx_snk_in_arr,  -- 8b beamlets

    src_out_arr                    => dp_offload_tx_src_out_arr,
    src_in_arr                     => dp_offload_tx_src_in_arr,

    ID                             => ID,

    reg_dp_offload_tx_mosi         => c_mem_mosi_rst,
    reg_dp_offload_tx_miso         => OPEN,
    reg_dp_offload_tx_hdr_dat_mosi => c_mem_mosi_rst,
    reg_dp_offload_tx_hdr_dat_miso => OPEN
  );
 
  u_mmm : ENTITY work.mmm_apertif_unb1_fn_bf_emu 
  GENERIC MAP(
    -- General
    g_sim         => g_sim,         
    g_sim_unb_nr  => g_sim_unb_nr,  
    g_sim_node_nr => g_sim_node_nr
  )
  PORT MAP (
    xo_clk         => xo_clk,
    xo_rst_n       => xo_rst_n,                    
    xo_rst         => xo_rst,                      

    mm_rst         => mm_rst,                      
    mm_clk         => mm_clk,                      
    mm_locked      => mm_locked,                   
    cal_clk        => cal_rec_clk,                     

    pout_wdi       => pout_wdi,                    
    
    -- Manual WDI override
    reg_wdi_mosi               => reg_wdi_mosi,                  
    reg_wdi_miso               => reg_wdi_miso,                  

    -- system_info             
    reg_unb_system_info_mosi   => reg_unb_system_info_mosi,      
    reg_unb_system_info_miso   => reg_unb_system_info_miso,      
    rom_unb_system_info_mosi   => rom_unb_system_info_mosi,      
    rom_unb_system_info_miso   => rom_unb_system_info_miso,      

    -- UniBoard I2C sensors    
    reg_unb_sens_mosi          => reg_unb_sens_mosi,              
    reg_unb_sens_miso          => reg_unb_sens_miso,   
    
    -- PPSH
    reg_ppsh_mosi              => reg_ppsh_mosi,
    reg_ppsh_miso              => reg_ppsh_miso,     


    -- Block gens
    ram_diag_bg_mosi           => ram_diag_bg_mosi,                 
    ram_diag_bg_miso           => ram_diag_bg_miso,                 
    reg_diag_bg_mosi           => reg_diag_bg_mosi,                 
    reg_diag_bg_miso           => reg_diag_bg_miso,                 

    reg_tr_10GbE_mosi              => reg_tr_10GbE_mosi,              
    reg_tr_10GbE_miso              => reg_tr_10GbE_miso,              

    reg_mdio_mosi_arr              => reg_mdio_mosi_arr,              
    reg_mdio_miso_arr              => reg_mdio_miso_arr,              

    reg_tr_xaui_mosi               => reg_tr_xaui_mosi,               
    reg_tr_xaui_miso               => reg_tr_xaui_miso,    
     
    -- eth1g     
    eth1g_tse_clk                  => eth1g_tse_clk,                  
    eth1g_mm_rst                   => eth1g_mm_rst,                   
    eth1g_tse_mosi                 => eth1g_tse_mosi,                 
    eth1g_tse_miso                 => eth1g_tse_miso,                 
    eth1g_reg_mosi                 => eth1g_reg_mosi,                 
    eth1g_reg_miso                 => eth1g_reg_miso,                 
    eth1g_reg_interrupt            => eth1g_reg_interrupt,
    eth1g_ram_mosi                 => eth1g_ram_mosi,                 
    eth1g_ram_miso                 => eth1g_ram_miso                 
  );
END;













