-------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_field_pkg.ALL;

PACKAGE apertif_udp_offload_pkg IS

  CONSTANT c_apertif_udp_offload_nof_hdr_fields : NATURAL := 3+12+4+3; -- 448b; 7 64b words
  -- Notes: 
  -- . pre-calculated ip_header_checksum is valid only for UNB0, FN0 targeting IP 10.10.10.10
  -- . Default ip_total_length and udp_total_length valid for 188 beamlets * 64b
  CONSTANT c_apertif_udp_offload_hdr_field_arr : t_common_field_arr(c_apertif_udp_offload_nof_hdr_fields-1 DOWNTO 0) := ( ( field_name_pad("eth_dst_mac"            ), "RW", 48, field_default(x"AABBCCDDEEFF") ),
                                                                                                                          ( field_name_pad("eth_src_mac"            ), "RW", 48, field_default(0) ),
                                                                                                                          ( field_name_pad("eth_type"               ), "RW", 16, field_default(x"0800") ),
                                                                                                                          ( field_name_pad("ip_version"             ), "RW",  4, field_default(4) ),
                                                                                                                          ( field_name_pad("ip_header_length"       ), "RW",  4, field_default(5) ),
                                                                                                                          ( field_name_pad("ip_services"            ), "RW",  8, field_default(0) ),
                                                                                                                          ( field_name_pad("ip_total_length"        ), "RW", 16, field_default(1546) ), 
                                                                                                                          ( field_name_pad("ip_identification"      ), "RW", 16, field_default(0) ),
                                                                                                                          ( field_name_pad("ip_flags"               ), "RW",  3, field_default(2) ),
                                                                                                                          ( field_name_pad("ip_fragment_offset"     ), "RW", 13, field_default(0) ),
                                                                                                                          ( field_name_pad("ip_time_to_live"        ), "RW",  8, field_default(127) ),
                                                                                                                          ( field_name_pad("ip_protocol"            ), "RW",  8, field_default(17) ),
                                                                                                                          ( field_name_pad("ip_header_checksum"     ), "RW", 16, field_default(55147) ),
                                                                                                                          ( field_name_pad("ip_src_addr"            ), "RW", 32, field_default(0) ),
                                                                                                                          ( field_name_pad("ip_dst_addr"            ), "RW", 32, field_default(x"0A0A0A0A") ),
                                                                                                                          ( field_name_pad("udp_src_port"           ), "RW", 16, field_default(0) ), 
                                                                                                                          ( field_name_pad("udp_dst_port"           ), "RW", 16, field_default(0) ), 
                                                                                                                          ( field_name_pad("udp_total_length"       ), "RW", 16, field_default(1526) ),
                                                                                                                          ( field_name_pad("udp_checksum"           ), "RW", 16, field_default(0) ),
                                                                                                                          ( field_name_pad("dp_reserved"            ), "RW", 47, field_default(0) ),
                                                                                                                          ( field_name_pad("dp_sync"                ), "RW",  1, field_default(0) ),
                                                                                                                          ( field_name_pad("dp_bsn"                 ), "RW", 64, field_default(0) ) );

END apertif_udp_offload_pkg;

PACKAGE BODY apertif_udp_offload_pkg IS
END apertif_udp_offload_pkg;

