 -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --

library IEEE;
use IEEE.STD_LOGIC_1164.all; 
use ieee.numeric_std.all;


entity it_mod is
   generic(
      C_MOD     : natural := 10;
      C_WIDTH   : natural := 10;
      C_INIT    : natural :=  1;
      C_STEP    : natural :=  1      
   );
   port (
      RST       : in  std_logic;
      CLK       : in  std_logic;
      LOAD      : in  std_logic;
      ENABLE    : in  std_logic;
      MODULE    : out std_logic_vector(C_WIDTH-1 downto 0)
   );
end it_mod;

architecture RTL of it_mod is
   signal sl_reg      : unsigned(C_WIDTH-1 downto 0);
   signal sl_step     : unsigned(C_WIDTH-1 downto 0) := TO_UNSIGNED(C_STEP,C_WIDTH);
   signal sl_init     : unsigned(C_WIDTH-1 downto 0) := TO_UNSIGNED(C_INIT,C_WIDTH);
   signal sl_mod      : unsigned(C_WIDTH-1 downto 0) := TO_UNSIGNED(C_MOD ,C_WIDTH);
--   signal a1          : unsigned(C_WIDTH  downto 0);
--   signal a2          : unsigned(C_WIDTH  downto 0);
begin 
   --
   -- Checks 
   assert (2**C_WIDTH >= C_MOD)
      report "Parameter C_MOD (=" & integer'image(C_MOD) & ") bitwidth exceeds the output bitwidth (C_WIDTH=" & integer'image(C_WIDTH) & ")"
      severity ERROR;
   --
   assert (C_MOD >= C_STEP)
      report "Parameter C_STEP (=" & integer'image(C_STEP) & ") greater than parameter C_MOD (=" & integer'image(C_MOD) & ")"
      severity ERROR;
   --
   assert (C_MOD >= C_INIT)
      report "Parameter C_INIT (=" & integer'image(C_INIT) & ") greater than parameter C_MOD (=" & integer'image(C_MOD) & ")"
      severity ERROR;

   
   --
   REG_PRCS : process(CLK, RST)
      variable a1 : unsigned(C_WIDTH  downto 0);
      variable a2 :   signed(C_WIDTH  downto 0);
   begin
      if rising_edge(CLK) then
         --if( RST='1' or LOAD='1') then
         if( RST='1' ) then
            sl_reg <= sl_init;
         elsif( ENABLE='1' ) then
            a1 := ("0"&sl_reg) + ("0"&sl_step);
            a2 := SIGNED(a1) - SIGNED("0"&sl_mod);
            if ( LOAD='1' ) then 
               sl_reg <= sl_init;
            elsif (a2(a2'High)='1') then
               sl_reg <= a1(C_WIDTH-1 downto 0);
            else
               sl_reg <= UNSIGNED(a2(C_WIDTH-1 downto 0));
            end if;
         end if;
      end if;
   end process;
   --
   MODULE <= STD_LOGIC_VECTOR(sl_reg);
   --
end RTL;

