 -- COPYRIGHT NOTICE (NOT TO BE REMOVED):
 --
 -- This file, or parts of it, or modified versions of it, may not be
 -- copied, reproduced or transmitted in any form, including
 -- reprinting, translation, photocopying or microfilming, or by any
 -- means, electronic, mechanical or otherwise, or stored in a
 -- retrieval system, or used for any purpose, without the prior
 -- written permission of all Owners unless it is explicitly marked as
 -- having Classification `Public'.
 --
 -- Classification: Restricted.
 --
 -- Owners of this file give notice:
 -- (c) Copyright 2005 - 2009 Compaan Design bv, The Netherlands
 -- All rights, including copyrights, reserved.
 --
 -- This file contains or may contain restricted information and is
 -- UNPUBLISHED PROPRIETARY SOURCE CODE OF THE Owners. The Copyright
 -- Notice(s) above do not evidence any actual or intended publication
 -- of such source code. This file is additionally subject to the
 -- conditions listed in the RESTRICTIONS file and is with NO WARRANTY.
 --
 -- END OF COPYRIGHT NOTICE
 --

library IEEE;
use IEEE.STD_LOGIC_1164.all; 
use ieee.numeric_std.all;


entity it_mul is
   generic(
      C_WIDTH   : natural := 10;
      C_INIT    : natural :=  1;
      C_STEP    : natural :=  1
   );
   port (
      RST       : in  std_logic;
      CLK       : in  std_logic;
      LOAD      : in  std_logic;
      ENABLE    : in  std_logic;
      MUL       : out std_logic_vector(C_WIDTH-1 downto 0)
   );
end it_mul;

architecture RTL of it_mul is
   signal sl_counter  : unsigned(C_WIDTH-1 downto 0);
   signal sl_step     : unsigned(C_WIDTH-1 downto 0) := TO_UNSIGNED(C_STEP,C_WIDTH);
   signal sl_init     : unsigned(C_WIDTH-1 downto 0) := TO_UNSIGNED(C_INIT,C_WIDTH);
begin 
   --
   -- Checks
   assert (2**C_WIDTH > C_INIT)
      report "Parameter C_INIT (=" & integer'image(C_INIT) & ") bitwidth exceeds the output bitwidth (C_WIDTH=" & integer'image(C_WIDTH) & ")"
      severity ERROR;
   --   
   assert (2**C_WIDTH > C_STEP)
      report "Parameter C_STEP (=" & integer'image(C_STEP) & ") bitwidth exceeds the output bitwidth (C_WIDTH=" & integer'image(C_WIDTH) & ")"
      severity ERROR;
   --
   REG_PRCS : process(CLK, RST)
   begin
      if rising_edge(CLK) then
         --if( RST='1' or LOAD='1') then
         if( RST='1' ) then
            sl_counter <= sl_init;
         elsif( ENABLE='1' ) then
            if ( LOAD='1') then
                sl_counter <= sl_init;
            else
                sl_counter <= sl_counter + sl_step;
            end if;
         end if;
      end if;
   end process;
   --
   MUL <= STD_LOGIC_VECTOR(sl_counter);
   --
end RTL;

