-- File automatically generated by KpnMapper
-- This file defines a template for pipelined function implementation
-- Function "transformer"

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.hw_node_pkg.all;

entity transformer_pipeline is
   generic (
      c_STAGES   : natural := 1;
      N_CNTRS    : natural := 1;
      CNTR_QUANT : natural := 32;
      CNTR_WIDTH : t_counter_width := ( 0=>10, 1=>10, 2=>9, others=>10 )
   );
   port (
      RST   : in  std_logic;
      CLK   : in  std_logic;
      -- Inputs 
      ip_a  : in  std_logic_vector(31 downto 0);
      -- Iterators 
      it_j : in  std_logic_vector(CNTR_WIDTH(0)-1 downto 0);
      -- Outputs 
      op_b : out std_logic_vector(31 downto 0);
      --
      ENi   : in  std_logic;
      EN    : in  std_logic_vector(c_STAGES-1 downto 0);
      STALL_FRONT  : out std_logic_vector(c_STAGES-1 downto 0);
      STALL_BACK   : out std_logic_vector(c_STAGES-1 downto 0);
      ERROR : out std_logic
   );
end transformer_pipeline;

architecture RTL of transformer_pipeline is
--
    constant error_int : integer := -1;
    constant reset_int : std_logic_vector(0 downto 0) := b"0";
    -- Input registers
    signal ipr_a : std_logic_vector(31 downto 0);

    -- Iterator registers 
    signal itr_j : std_logic_vector(CNTR_WIDTH(0)-1 downto 0);
    --

    --
    -- Your pipeline signals
    --
    -- STAGE_0
    signal s0_a : std_logic_vector(31 downto 0);
    signal r0_a : std_logic_vector(31 downto 0);

begin

    PIPE_REGS : process(CLK)
    begin
        if rising_edge(CLK) then
            if (RST='1') then
                -- Something to reset?
            else
                if( ENi = '1' ) then
                    -- Input Registers 
                    ipr_a <= ip_a;
                    -- Iterator Registers 
                    itr_j <= it_j;
                end if;
                -- Pipeline Depth: 1 stages
                -- STAGE_0
                if( EN(0) = '1' ) then
                    r0_a <= s0_a(30 downto 0) & '0';
                end if;
            end if;
        end if;
    end process;    -- PIPE_REGS
    --
    -- Output
    op_b <= STD_LOGIC_VECTOR(RESIZE(UNSIGNED(r0_a), op_b'Length));
    --
    -- PIPE_COMB:
    s0_a <= ipr_a;
    --
    STALL_FRONT  <= (others=>'0');
    STALL_BACK   <= (others=>'0');
    ERROR <= '0';
end RTL;


