-- Execute Unit automatically generated by KpnMapper
-- Function "transformer"

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.hw_node_pkg.all;

entity ipcore2RTL_EXECUTION_UNIT_hwn_nd_2 is
   generic (
      N_INPORTS  : natural := 1;
      N_OUTPORTS : natural := 1;
      IP_RESET   : natural := 1; 
      QUANT      : natural := 32;
      c_STAGES   : natural := 1;
      N_CNTRS    : natural := 1; 
      CNTR_QUANT : natural := 32; 
      CNTR_WIDTH : t_counter_width := ( 0=>10, 1=>10, 2=>9, others=>10 ) 
   );
   port (
      RST        : in  std_logic;
      CLK        : in  std_logic;
      -- Funtion Input parameters
      IN_PORT_0    : in  std_logic_vector(31 downto 0);  -- Param. "a"
      READ       : out std_logic_vector(N_INPORTS-1 downto 0);
      EXIST      : in  std_logic_vector(N_INPORTS-1 downto 0);
      -- Iterators
      REG_CNTRS_RD : in  std_logic_vector(N_CNTRS*CNTR_QUANT-1 downto 0);
      -- Funtion Output parameters
      OUT_PORT_0   : out std_logic_vector(31 downto 0);  -- Param. "b"
      WRITE      : out std_logic_vector(N_OUTPORTS-1 downto 0);
      FULL       : in  std_logic_vector(N_OUTPORTS-1 downto 0);
      STOP_RD    : in  std_logic;
      STOP_WR    : in  std_logic;
      ERROR      : out std_logic
   );
end ipcore2RTL_EXECUTION_UNIT_hwn_nd_2 ;

-- Laura implementation
architecture Laura of ipcore2RTL_EXECUTION_UNIT_hwn_nd_2 is

   component transformer is
      generic (
         c_STAGES   : natural := 1;
         N_CNTRS    : natural := 1; 
         CNTR_QUANT : natural := 32; 
         CNTR_WIDTH : t_counter_width := ( 0=>10, 1=>10, 2=>9, others=>10 ) 
      );
      port (
         RST   : in std_logic;
         CLK   : in std_logic;
         -- Inputs 
         ip_a  : in  std_logic_vector(31 downto 0);  
         -- Iterators 
         it_j : in  std_logic_vector(CNTR_WIDTH(0)-1 downto 0);
         EXIST : in  std_logic_vector(0 downto 0);
         READF : out std_logic_vector(0 downto 0);
         -- Outputs 
         op_b : out std_logic_vector(31 downto 0);
         FULL  : in  std_logic_vector(0 downto 0);
         WRITEF: out std_logic_vector(0 downto 0);
         --
         STOP_RD : in  std_logic;
         STOP_WR : in  std_logic;
         ERROR   : out std_logic
      );
   end component;

   signal sl_RST : std_logic;

begin

   sl_RST <= RST when IP_RESET=1 else not RST;

   FUNC : transformer
   generic map (
         c_STAGES      => c_STAGES,
         N_CNTRS       => N_CNTRS,
         CNTR_QUANT    => CNTR_QUANT,
         CNTR_WIDTH    => CNTR_WIDTH
   )
   port map (
      RST   => sl_RST,
      CLK   => CLK,
      -- Inputs 
      ip_a => IN_PORT_0,
      -- Iterators 
      it_j => REG_CNTRS_RD(0*CNTR_QUANT+CNTR_WIDTH(0)-1 downto 0*CNTR_QUANT),
      EXIST => EXIST,
      READF => READ,
      -- Outputs 
      op_b => OUT_PORT_0,
      FULL  => FULL,
      WRITEF=> WRITE,
      -- 
      STOP_RD => STOP_RD,
      STOP_WR => STOP_WR,
      ERROR   => ERROR
   );

end Laura;
