-------------------------------------------------------------------------------
-- ipcore2RTL_ed_1_wrapper.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

entity pkg_signals is    
  generic (
    BLOCKS_PER_SYNC	: natural := 10
  );
	port (
		write_in		: in std_logic;
		data_in 	  : in std_logic_vector(31 downto 0);
		control_in  : in std_logic;
		write_out 	: out std_logic;
		data_out 	  : out std_logic_vector(31 downto 0);
		control_out : out std_logic;
		eop_out  	  : out std_logic;
		sop_out		  : out std_logic;
		RST 				: in std_logic;
		CLK 				: in std_logic
	);
end pkg_signals;

architecture BEHAVIOURAL of pkg_signals is

  signal sync_count : natural;
  signal sl_eop : std_logic;
  signal sl_sop : std_logic;
  
	type state_type is (s_idle, s_count);
	signal sync_state : state_type;  

begin
 
  sop_out <= sl_sop;
  eop_out <= sl_eop;

  pkg_signals_gen : PROCESS(CLK)
  begin
    if rising_edge(CLK) then
      if RST = '1' then
        sl_sop <= '0';
        sl_eop <= '0';
        sync_state <= s_idle;
        sync_count <= 0;
      else
        write_out <= write_in;
        data_out <= data_in;
        control_out <= control_in;
        sync: case sync_state is
          when s_idle => 
            sl_sop <= write_in;
            sl_eop <= '0';
            if write_in = '1' then
              sync_state <= s_count; 
              sync_count <= 0;
            end if;
          when s_count =>    
            sl_sop <= '0';
						if sync_count < BLOCKS_PER_SYNC - 2 then
							sync_count <= sync_count + 1;
						else
	            sl_eop <= '1';
	            sync_state <= s_idle;
	          end if;
	        when others =>
	          sync_state <= s_idle;
        end case;
      end if;
    end if;
  end process;

end architecture;

