-------------------------------------------------------------------------------
--
-- Copyright (C) 2015
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, technology_lib, tech_pll_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE technology_lib.technology_pkg.ALL;

-- Purpose: PLL for UniBoard node CLK input @ 25 MHz
-- Description:
--   c0 = 20 MHz
--   c1 = 40 MHz
--   c2 = 50 MHz
--   c3 = 125 MHz
-- 

ENTITY unb1_board_clk25_pll IS
  GENERIC (
    g_technology : NATURAL := c_tech_stratixiv
  );
  PORT (
    arst        : IN  STD_LOGIC := '0';
    clk25       : IN  STD_LOGIC := '0'; -- connect to UniBoard ETH_clk pin (25 MHz)

    c0_clk20    : OUT STD_LOGIC;  -- PLL c0
    c1_clk40    : OUT STD_LOGIC;  -- PLL c1
    c2_clk50    : OUT STD_LOGIC;  -- PLL c2
    c3_clk125   : OUT STD_LOGIC;  -- PLL c3
    pll_locked  : OUT STD_LOGIC
  );
END unb1_board_clk25_pll;


ARCHITECTURE stratixiv OF unb1_board_clk25_pll IS
BEGIN

  u_pll : ENTITY tech_pll_lib.tech_pll_clk25
  GENERIC MAP (
    g_technology => g_technology
  )
  PORT MAP (
    areset  => arst,
    inclk0  => clk25,
    c0      => c0_clk20,
    c1      => c1_clk40,
    c2      => c2_clk50,
    c3      => c3_clk125,
    locked  => pll_locked
  );
END stratixiv;
