#!/bin/bash
# -------------------------------------------------------------------------- #
#
# Copyright (C) 2014                                                        
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>           
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands                             
#                                                                           
# This program is free software: you can redistribute it and/or modify      
# it under the terms of the GNU General Public License as published by      
# the Free Software Foundation, either version 3 of the License, or         
# (at your option) any later version.                                       
#                                                                           
# This program is distributed in the hope that it will be useful,           
# but WITHOUT ANY WARRANTY; without even the implied warranty of            
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
# GNU General Public License for more details.                              
#                                                                           
# You should have received a copy of the GNU General Public License         
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
# -------------------------------------------------------------------------- #
#
# Purpose: Generate phy_xaui IP with MegaWizard
# Description:
#   Generate the phy_xaui IP in a separate generated/ subdirectory. Therefore first copy the MegaWizard variation file in the 'generated' output dir, because:
#   1) setting the output dir is not an option in the MegaWizard
#   2) The MegaWizard overwrites the variation file sometimes - we don't want that as it could be hand-modified with e.g. added generics
#
# Usage:
#
#   ./generate_ip.sh
#
#   The generated files for Quartus v11.1 that are needed for simulation and synthesis have been committed in SVN, so it is not necessary to run this
#   generate_ip.sh script again. Files that are not needed have not been committed in SVN.
#
# Remarks:
# . This dedicated script is Derived from the generic script $UNB/Firmware/software/build/unb_mgw
#

# Tool settings for selected target "unb1" with stratixiv
. ${RADIOHDL}/tools/quartus/set_quartus unb1

if ! [ -d "generated" ]; then
    mkdir generated
fi
cd generated
cp ../ip_stratixiv_phy_xaui_0.vhd .
cp ../ip_stratixiv_phy_xaui_1.vhd .
cp ../ip_stratixiv_phy_xaui_2.vhd .
cp ../ip_stratixiv_phy_xaui_soft.vhd .

qmegawiz -silent -f:$UNB/Firmware/software/build/unb_mgw_params.txt ip_stratixiv_phy_xaui_0.vhd
qmegawiz -silent -f:$UNB/Firmware/software/build/unb_mgw_params.txt ip_stratixiv_phy_xaui_1.vhd
qmegawiz -silent -f:$UNB/Firmware/software/build/unb_mgw_params.txt ip_stratixiv_phy_xaui_2.vhd
qmegawiz -silent -f:$UNB/Firmware/software/build/unb_mgw_params.txt ip_stratixiv_phy_xaui_soft.vhd

rm ip_stratixiv_phy_xaui_0.vhd
rm ip_stratixiv_phy_xaui_1.vhd
rm ip_stratixiv_phy_xaui_2.vhd
rm ip_stratixiv_phy_xaui_soft.vhd
