-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Reusable test bench end control.
-- Description:
--    Especially suitable for the tb that reuse the tb_tech_mac_10g_* stimuli
--    components and verification components.

LIBRARY IEEE, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;


ENTITY tb_tech_mac_10g_simulation_end IS
  GENERIC (
    g_tb_end            : BOOLEAN := TRUE;
    g_nof_clk_to_rx_end : NATURAL := 1000
  );
  PORT (
    clk       : IN  STD_LOGIC;
    tx_end    : IN  STD_LOGIC;
    rx_end    : OUT STD_LOGIC;
    tb_end    : OUT STD_LOGIC
  );
END tb_tech_mac_10g_simulation_end;

ARCHITECTURE tb OF tb_tech_mac_10g_simulation_end IS
BEGIN

  p_tb_end : PROCESS  
  BEGIN
    tb_end <= '0';
    rx_end <= '0';
    WAIT UNTIL tx_end='1';
    proc_common_wait_some_cycles(clk, g_nof_clk_to_rx_end);
    rx_end <= '1';
    proc_common_wait_some_cycles(clk, 100);
    
    -- Stop the simulation
    tb_end <= '1';
    IF g_tb_end=FALSE THEN
      REPORT "Tb simulation finished." SEVERITY NOTE;
    ELSE
      REPORT "Tb simulation finished." SEVERITY FAILURE;
    END IF;
    WAIT;
  END PROCESS;

END tb;
