###############################################################################
#
# Copyright (C) 2014
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

"""

   
"""


import common as cm
import common_dict_file
import sys
import os
import os.path
import shutil
import argparse
import vhdl
import hdl_config

s_header = """--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

"""

s_libraries = "LIBRARY IEEE, common_lib, unb1_board_lib, mm_lib, eth_lib, technology_lib, tech_tse_lib;\n\
USE IEEE.STD_LOGIC_1164.ALL;\n\
USE IEEE.NUMERIC_STD.ALL;\n\
USE common_lib.common_pkg.ALL;\n\
USE common_lib.common_mem_pkg.ALL;\n\
USE common_lib.tb_common_mem_pkg.ALL;\n\
USE common_lib.common_field_pkg.ALL;\n\
USE common_lib.common_network_total_header_pkg.ALL;\n\
USE common_lib.common_network_layers_pkg.ALL;\n\
USE common_lib.common_mem_pkg.ALL;\n\
USE unb1_board_lib.unb1_board_pkg.ALL;\n\
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;\n\
USE mm_lib.mm_file_pkg.ALL;\n\
USE mm_lib.mm_file_unb_pkg.ALL;\n\
USE eth_lib.eth_pkg.ALL;\n\
USE technology_lib.technology_pkg.ALL;\n\
USE tech_tse_lib.tech_tse_pkg.ALL;\n\
USE tech_tse_lib.tb_tech_tse_pkg.ALL;\n\n"

s_signals_clk = "SIGNAL i_mm_clk   : STD_LOGIC := '1';\n\
SIGNAL i_dp_clk   : STD_LOGIC := '1';\n"

s_connect_mm_clk = "mm_clk   <= i_mm_clk;\n"
s_connect_dp_clk = "dp_clk   <= i_dp_clk;\n"

s_fileio_start = "----------------------------------------------------------------------------\n\
-- MM <-> file I/O for simulation. The files are created in $UPE/sim.\n\
----------------------------------------------------------------------------\n\
gen_mm_file_io : IF g_sim = TRUE GENERATE\n"

s_fileio_eth_init = "----------------------------------------------------------------------------\n\
-- 1GbE setup sequence normally performed by unb_os@NIOS\n\
----------------------------------------------------------------------------\n\
p_eth_setup : PROCESS\n\
BEGIN\n\
  sim_eth_mm_bus_switch <= '1';\n\
  eth1g_tse_mosi.wr <= '0';\n\
  eth1g_tse_mosi.rd <= '0';\n\
  WAIT FOR 400 ns;\n\
  WAIT UNTIL rising_edge(mm_clk);\n\
  proc_tech_tse_setup(c_tech_stratixiv, FALSE, c_tech_tse_tx_fifo_depth, c_tech_tse_rx_fifo_depth, c_tech_tse_tx_ready_latency, c_sim_eth_src_mac, sim_eth_psc_access, mm_clk, eth1g_tse_miso, eth1g_tse_mosi);\n\
  -- Enable RX\n\
  proc_mem_mm_bus_wr(c_eth_reg_control_wi+0, c_sim_eth_control_rx_en, mm_clk, eth1g_reg_miso, sim_eth1g_reg_mosi);  -- control rx en\n\
  sim_eth_mm_bus_switch <= '0';\n\
  WAIT;\n\
END PROCESS;\n\
\n\
p_switch : PROCESS(sim_eth_mm_bus_switch, sim_eth1g_reg_mosi, i_eth1g_reg_mosi)\n\
BEGIN\n\
  IF sim_eth_mm_bus_switch = '1' THEN\n\
    eth1g_reg_mosi <= sim_eth1g_reg_mosi;\n\
  ELSE\n\
    eth1g_reg_mosi <= i_eth1g_reg_mosi;\n\
  END IF;\n\
END PROCESS;\n"

s_fileio_poll = "----------------------------------------------------------------------------\n\
-- Procedure that polls a sim control file that can be used to e.g. get\n\
-- the simulation time in ns\n\
----------------------------------------------------------------------------\n\
mmf_poll_sim_ctrl_file(c_mmf_unb_file_path & \"sim.ctrl\", c_mmf_unb_file_path & \"sim.stat\");\n"

s_end_generate = "\nEND GENERATE;"

s_master_header = "----------------------------------------------------------------------------\n\
-- SOPC or QSYS for synthesis\n\
----------------------------------------------------------------------------\n"

s_arch_begin = "\nBEGIN\n\n"
s_arch_end   = "\nEND str;\n"

peripherals_vhdl2python = { 'reg_wdi'             : "reg_wdi",\
                            'reg_unb_system_info' : "pio_system_info",\
                            'rom_unb_system_info' : "rom_system_info",\
                            'reg_unb_sens'        : "reg_unb_sens",\
                            'reg_ppsh'            : "pio_pps",\
                            'eth1g_reg'           : "avs_eth_0_mms_reg"}

class QsysEntity(vhdl.Entity): 

    def __init__(self, name= ""):  
        vhdl.Entity.__init__(self, name)
    
    def make_qsys_instance_string(self, mmm_conf): 
        s_start = "u_" + self.name + " : " + self.name + "\n" + self.s_port_map_start 
        self.portNames.sort()
        s_port_connects = ""
        for s in self.portNames: 
            s_connect = ""
            if "_address_export" in s:  
                s_reg_name = self.name_convert(s.split("_address_export")[0])
                if s_reg_name[-1].isdigit():
                    s_index    = s_reg_name.rsplit("_", 1)[1]
                    s_reg_name = s_reg_name.rsplit("_", 1)[0]
                    s_connect =  s_reg_name + "_mosi_arr(" + s_index + ").address("
                else:
                    s_connect =  s_reg_name + "_mosi.address("
                    
                span = self.find_span(s_reg_name, mmm_conf.peripherals) 
                if span == 0: 
                    sys.exit('Error: No corresponding register in mmm.cfg for QSYS register: ' + s_reg_name)
                if span == 1: 
                    s_connect = s_connect + "0),"
                else:
                    s_connect = s_connect + str(span-1) + " DOWNTO 0),"
            
            if "_read_export" in s:  
                s_reg_name = self.name_convert(s.split("_read_export")[0])
                if s_reg_name[-1].isdigit():
                    s_index    = s_reg_name.rsplit("_", 1)[1]
                    s_reg_name = s_reg_name.rsplit("_", 1)[0]
                    s_connect =  s_reg_name + "_mosi_arr(" + s_index + ").rd,"
                else:
                    s_connect =  s_reg_name + "_mosi.rd,"

            if "_write_export" in s:  
                s_reg_name = self.name_convert(s.split("_write_export")[0])
                if s_reg_name[-1].isdigit():
                    s_index    = s_reg_name.rsplit("_", 1)[1]
                    s_reg_name = s_reg_name.rsplit("_", 1)[0]
                    s_connect =  s_reg_name + "_mosi_arr(" + s_index + ").wr,"
                else:
                    s_connect =  s_reg_name + "_mosi.wr,"

            if "_readdata_export" in s:  
                s_reg_name = self.name_convert(s.split("_readdata_export")[0])
                if s_reg_name[-1].isdigit():
                    s_index    = s_reg_name.rsplit("_", 1)[1]
                    s_reg_name = s_reg_name.rsplit("_", 1)[0]
                    s_connect =  s_reg_name + "_miso_arr(" + s_index + ").rddata(c_word_w-1 DOWNTO 0),"
                else:
                    s_connect =  s_reg_name + "_miso.rddata(c_word_w-1 DOWNTO 0),"

            if "_writedata_export" in s:  
                s_reg_name = self.name_convert(s.split("_writedata_export")[0])
                if s_reg_name[-1].isdigit():
                    s_index    = s_reg_name.rsplit("_", 1)[1]
                    s_reg_name = s_reg_name.rsplit("_", 1)[0]
                    s_connect =  s_reg_name + "_mosi_arr(" + s_index + ").wrdata(c_word_w-1 DOWNTO 0),"
                else:
                    s_connect =  s_reg_name + "_mosi.wrdata(c_word_w-1 DOWNTO 0),"

            if "_clk_export" in s:  
                s_connect =  "OPEN,"

            if "_reset_export" in s:  
                s_connect =  "OPEN,"
                
            # Exceptions for the ethernet
            if "tse_clk" in s:  
                s_connect =  "eth1g_tse_clk,"

            if "eth1g_mm_rst_export" in s:  
                s_connect =  "eth1g_mm_rst,"

            if "eth1g_irq_export" in s:  
                s_connect =  "eth1g_reg_interrupt,"

            if "eth1g_tse_waitrequest_export" in s:  
                s_connect =  "eth1g_tse_miso.waitrequest,"

            # Exceptions for the debug_wave
            if "out_port_from_the_pio_debug_wave" in s:  
                s_connect =  "OPEN,"

            # Exceptions for the debug_wave
            if "out_port_from_the_pio_wdi" in s:  
                s_connect =  "pout_wdi,"

            # mm_clk 
            if "clk_in_clk" in s:  
                s_connect =  "mm_clk,"

            if s == "reset_in_reset_n":  
                s_connect =  "mm_rst,"

            if "_phasedone_export" in s:  
                s_connect =  "OPEN,"

            s_port_connects = s_port_connects + "  " + s.ljust(self.longestPortName) + " => " + s_connect + "\n"
        
        s_port_connects = s_port_connects.rsplit(",",1)[0] + "\n"
        
        return s_start + s_port_connects + ");"
                                         
    def name_convert(self, name):
        new_name = name  
        for reg_name, sopc_name in peripherals_vhdl2python.items():
            if sopc_name == name:
               new_name = reg_name
        return new_name
    
    def find_span(self, name, peripherals):
        """Function find_span calculates te address span based on the values in column 1 and 2 of a peripheral entry.
           Column 1 (C1) defines the number of instances.
           Column 2 (C2) defines the address span of single instance.
           The total address span is ceil_log2(C1) + C2. 
        """
        span = 0 
        for i in range(len(peripherals)):
            if peripherals[i][0] == name:
                span = cm.ceil_log2(int(peripherals[i][1])) + int(peripherals[i][2])
        return span  

class MmmEntity(vhdl.Entity): 
    
    def __init__(self, mmm_conf):  
        vhdl.Entity.__init__(self, mmm_conf.mmmName)
        # Add default generics
        self.add_generic( "g_sim",         "BOOLEAN", "FALSE")
        self.add_generic( "g_sim_unb_nr",  "NATURAL", "0")
        self.add_generic( "g_sim_node_nr", "NATURAL", "0")
        
        # Add input clks
        if "mm_clk" in mmm_conf.input_clks:
            self.add_port("mm_clk",     "IN", "STD_LOGIC", "\'1\'")      
            self.add_port("mm_rst",     "IN", "STD_LOGIC", "\'1\'")              
        
        # Add UNB1 board peripherals
        for s in mmm_conf.peripherals:   
            
            # Extra signals for ethernet peripheral
            if(s[0] == "eth1g_ram"): 
                self.add_port("eth1g_tse_clk",       "OUT", "STD_LOGIC")      
                self.add_port("eth1g_mm_rst",        "OUT", "STD_LOGIC")    
                self.add_port("eth1g_reg_interrupt", "IN", "STD_LOGIC")   
            
            # Extra signal for Watchdog interface
            if(s[0] == "reg_wdi"):                              
                self.add_port("pout_wdi", "OUT",  "STD_LOGIC", "\'1\'") 
                
            self.add_port(s[0] + "_mosi",     "OUT", "t_mem_mosi")      
            self.add_port(s[0] + "_miso",     "IN", "t_mem_miso", "c_mem_miso_rst") 
        

class MmmGenerate:

    def __init__(self, mmmRootDir, mmmFileName='mmm.cfg', mmmLibraryName=''):
        """
        """
        self.mmmRootDir = mmmRootDir
        self.mmmFileName = mmmFileName
        self.mmmLibraryName = mmmLibraryName
        # mmm config files
        self.mmm = common_dict_file.CommonDictFile(mmmRootDir, mmmFileName)        # library dict files
        if self.mmm.nof_dicts==0: sys.exit('Error : No mmm config file found')
        if self.mmmLibraryName=='': sys.exit('Error: No mmmLibraryName specified');
        
        for d in self.mmm.dicts:
            name = self.mmm.get_key_value('mmm_name', d)
            if name == mmmLibraryName:
                self.mmm.remove_all_but_the_dict_from_list(d)
        
        self.board_select = self.mmm.get_key_values('board_select')
        
        if self.board_select == 'unb1':
            self.board_peripherals_path = mmmRootDir + "/boards/uniboard1/libraries/unb1_board/unb1_board_peripherals.cfg"
        elif self.board_select == 'unb2':
            self.board_peripherals_path = mmmRootDir + "/boards/uniboard2/libraries/unb2_board/unb2_board_peripherals.cfg"
        
        self.board_peripherals_dict = self.mmm.read_dict_file(self.board_peripherals_path , ' ')
        self.board_peripherals      = self.chunks(self.mmm.get_key_values('board_peripherals', dicts=self.board_peripherals_dict).split(), 3)
        
        # Keep list of HDL library names
        self.designName                  = self.mmm.get_key_values('mmm_name')
        self.mmmName                     = "mmm_" + self.mmm.get_key_values('mmm_name')
        self.VhdlOutputPath              = cm.expand_file_path_name(self.mmm.get_key_values('vhdl_output_path'), self.mmm.filePaths[0])
        self.VhdlFileName                = self.mmmName + '.vhd'
        self.QsysName                    = "qsys_" + self.designName
        self.QsysFileName                = self.QsysName + '.vhd'
        self.input_clks                  = self.mmm.get_key_values('input_clks') 
        self.custom_peripherals_temp     = self.mmm.get_key_values('custom_peripherals').split() 
        self.custom_peripherals          = self.chunks(self.custom_peripherals_temp, 3)
        self.peripherals                 = self.board_peripherals + self.custom_peripherals
         
    def chunks(self, l, n):
        n = max(1, n)
        return [l[i:i + n] for i in range(0, len(l), n)]        
        
    def make_mm_file_io_entry_string(self, peripheral):        
        if peripheral[0] in peripherals_vhdl2python:
            python_name = peripherals_vhdl2python[peripheral[0]].upper()
        else:
            python_name = peripheral[0].upper()
        file_io_entry = "u_mm_file_" + peripheral[0] + "  :  mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & " + r'"' + python_name + r'"' + ")\n      PORT MAP(mm_rst, mm_clk, " + peripheral[0] + "_mosi, " + peripheral[0] + "_miso );\n"
        return file_io_entry
    
if __name__ == '__main__':

    # Parse command line arguments
    toolsetSelect = ['unb1', 'unb2']
    
    argparser = argparse.ArgumentParser(description='Create Memory Mapped Master entity for specified design name')
    argparser.add_argument('-t','--toolset', help='choose toolset %s (default: %s)' % (toolsetSelect,toolsetSelect[0]), default=toolsetSelect[0], required=False)
    argparser.add_argument('-v','--verbosity', help='verbosity >= 0 for more info', type=int, default=0, required=False)
    argparser.add_argument('-l','--library', help='specify the library name of the design', default='', required=True)
    args = vars(argparser.parse_args())
    arg_verbosity = args['verbosity']
    arg_library   = args['library']
    arg_toolset   = args['toolset']
    if arg_toolset not in toolsetSelect:
        print 'Toolset %s is not supported' % arg_toolset
        print 'Hint: give argument -h for possible options'
        sys.exit(1)
    toolFileName = 'hdltool_' + arg_toolset + '.cfg'

    designName  = arg_library # "unb1_terminal_bg_mesh_db"
    libRootDir  = 'RADIOHDL'
    mmmRootDir  = os.environ['RADIOHDL']
    mmmFileName = 'mmm.cfg'
    mm_dir      = '../../../../../UniBoard/trunk/Firmware/modules/mm/tb/vhdl/'
    mm_file     = "mm_file"
    
    mmm_conf = MmmGenerate( mmmRootDir = mmmRootDir, mmmFileName=mmmFileName, mmmLibraryName = designName)
    QsysBuildDir = os.environ['RADIOHDL']+'/build/' + arg_toolset + '/quartus/' + designName +'/qsys_' + designName + '/simulation/'

    ###################################    
    # Define the entity and architecture 
    ###################################
    ent_mmm     = MmmEntity(mmm_conf) 
    ent_qsys    = QsysEntity()
    ent_mm_file = vhdl.Entity()
    arch_mmm    = vhdl.Architecture(archName = "str", entityName = ent_mmm.name)

    ###################################    
    # Read the Qsys entity definition
    ###################################
    ent_qsys.read_entity_from_file(QsysBuildDir, mmm_conf.QsysName)
    ent_qsys.replace_std_logic_vector_with_std_logic()
   
    ###################################    
    # Read the mm_file entity definition
    ###################################
    ent_mm_file.read_entity_from_file(mm_dir, mm_file)

    ###################################    
    # Make the entity definition and a 
    # commented instantiation. 
    ###################################
    s_entity = ent_mmm.make_entity_definition_string()
    s_instantiation = ent_mmm.comment(ent_mmm.make_instantiation_string())

    ###################################    
    # Make the architecture header
    ###################################  
    arch_constants = [['c_sim_node_type',         'STRING(1 TO 2)',                                "sel_a_b(g_sim_node_nr<4, " + r'"FN", "BN"' + ")"],  
                      ['c_sim_node_nr',           'NATURAL',                                       "sel_a_b(c_sim_node_type=" + r'"BN"' +", g_sim_node_nr-4, g_sim_node_nr)"], 
                      ['c_sim_eth_src_mac',       "STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE)", 'X\"00228608\" & TO_UVEC(g_sim_unb_nr, c_byte_w) & TO_UVEC(g_sim_node_nr, c_byte_w)'], 
                      ['c_sim_eth_control_rx_en', 'NATURAL',                                       '2**c_eth_mm_reg_control_bi.rx_en']] 

    arch_signals = [['sim_eth_mm_bus_switch', 'STD_LOGIC',  ''],
                    ['sim_eth_psc_access',    'STD_LOGIC',  ''],
                    ['i_eth1g_reg_mosi',      't_mem_mosi', ''],
                    ['i_eth1g_reg_miso',      't_mem_miso', ''],
                    ['sim_eth1g_reg_mosi',    't_mem_mosi', '']]

    for s in arch_constants:
        arch_mmm.add_constant(s[0], s[1], s[2])

    for s in arch_signals:
        arch_mmm.add_signal(s[0], s[1], s[2])
        
    arch_mmm.add_component(ent_mm_file)
    arch_mmm.add_component(ent_qsys)
    
    s_arch_header = arch_mmm.make_arch_header_string()

    ###################################    
    # Make the architecture content
    ###################################
    s_arch_file = ""
    for s in mmm_conf.peripherals:
        s_arch_file = s_arch_file + mmm_conf.make_mm_file_io_entry_string(s)

    # Make File I/O master
    s_file    = s_fileio_start + ent_qsys.apply_indend(s_arch_file + s_fileio_eth_init + s_fileio_poll, 2) + s_end_generate + "\n"

    # Make QSYS master
    s_master_start = s_master_header + "gen_" + ent_qsys.name + " : IF g_sim = FALSE GENERATE\n" 
    s_master = s_master_start + ent_qsys.apply_indend(ent_qsys.make_qsys_instance_string(mmm_conf), 2) + s_end_generate + "\n"

    s_arch_total = s_arch_begin + ent_qsys.apply_indend(s_file + s_master, 2) + s_arch_end
    string_elements = [s_header, s_instantiation, s_libraries, s_entity, s_arch_header, s_arch_total ]
    
    ###################################    
    # Write the whole string to a file
    ###################################
    fileName = mmm_conf.VhdlOutputPath + mmm_conf.VhdlFileName
    f = file(fileName, "w")
    for s in string_elements:
        f.write(s)        
    f.close()
  